#include "USR_IO.h"

#define RT_NULL                         0
#define RT_EOK                          0               /**< There is no error */
#define RT_ERROR                        1               /**< A generic error happens */

#define rt_inline                   static __inline

//  MODBUS指令集
#define IO_READ_COIL                    0x01             // 读取读写开关（读线圈）
#define IO_READ_INPUT                   0x02            // 读取输入开关（读离散量输入）
#define IO_READ_HOLDING_REGISTER        0x03 // 读取读写寄存器（读保持寄存器）
#define IO_READ_INPUT_REGISTER          0x04   // 读取只读寄存器（读输入寄存器）
#define IO_WRITE_COIL                   0x05            // 写单个开关（写单个线圈）
#define IO_WRITE_REGISTER               0x06        // 写单个寄存器（写单个寄存器）
#define IO_WRITE_COIL_ANY               0x0F        // 写多个开关（写多个线圈）
#define IO_WRITE_REG_ANY                0x10         // 写多个寄存器（写多个寄存器）
#define WRITE_PRODUCT_REGISTER          0x41     //写生产参数寄存器
#define READ_FCAI_REGISTER              0x42     //生产读AI专用

#define RX2PIN GPIO_NUM_34
#define TX2PIN GPIO_NUM_13
#define TX2DEPIN GPIO_NUM_14

#define PROTOCOL_ADD_LEN     5
#define STATE_GET_DATA_LEN   1

#define ADDR_CODED_BUS_PROTOCOL_HEAD    0XDD
#define ADDR_CODED_BUS_DATA_LEN         12

#define EXT_POLL_TIMEOUT                50   //(MS)
#define EXT_POLL_TIME                   100   //(MS)
#define EXT_DI_PARAA_SET_TIMEOUT        5    //(S)
#define EXT_STATE_GET_TIMEOUT           3    //(S)
#define EXT_RESOURCES_GET_TIMEOUT       10    //(S)


/*extender_mng.h*/
/*给从机分配地址*/
#define EXT_BUAD        921600
#define EXT_CONFIG_BUAD   9600
#define EXT_ADDR        1
#define EXT_DATASIZE    8
#define EXT_STOPBIT     1
#define EXT_PARITY      0

enum ext_led_sta
{
    LED_ERR_MODE      = 1,     
    LED_WORT_MODE     = 2,
};

#define LED_ON     (1)
#define LED_OFF    (0)

#define MAX_SLOT_NUM  7        //1个M100主机 + 最大6个扩展机
#define MIN_SLOT_NUM  1        //最小槽位编号
#define FIX_SLOT_NUM  0        //M100主机固定分配槽位0

// typedef unsigned int           uint32_t;
// typedef unsigned short     int uint16_t;
// typedef unsigned          char uint8_t;

#ifndef DEV_TYPE
typedef unsigned int DEV_TYPE;
#endif

typedef enum
{
    USR_DEV_NULL = 0,
    USR_IO4040_EXT,      //串口IO，RS485，4DI,4DO  扩展机返回类型
    USR_IO0440_EXT,      //串口IO，RS485，4AI,4DO  扩展机返回类型
    USR_IO0080_EXT,      //串口IO，RS485，8DO      扩展机返回类型
    USR_IO8000_EXT,      //串口IO，RS485，8DI      扩展机返回类型
    USR_IO0800_EXT,      //串口IO，RS485，8AI      扩展机返回类型
    USR_IO0404_EXT,      //串口IO，RS485，4AI+4AO  扩展机返回类型6
    USR_M100 = 0x100,    //M100边缘网关
    USR_IO4040,          //串口IO，RS485，4DI,4DO
    USR_IO0440,          //串口IO，RS485，4AI,4DO
    USR_IO0080,          //串口IO，RS485，8DO
    USR_IO8000,          //串口IO，RS485，8DI
    USR_IO0800,          //串口IO，RS485，8AI
    USR_IO0404,          //串口IO，RS485，4AI+4AO
    USR_DEV_UNKNOW
}USR_DEV_E;


unsigned short usrio_module_slot1_usrio_type;
unsigned short usrio_module_slot2_usrio_type;
unsigned short usrio_module_slot3_usrio_type;




enum pro_reply_order
{
    PRO_ORDER_REPLY_CORRECT = 0x80,
    PRO_ORDER_REPLY_ERR  //协议字段未定义，预留
};

enum pro_reply_type
{
    PRO_REPLY_SUCCESS       = 0,
    PRO_REPLY_ERR_SUM       = 1,
    PRO_REPLY_ERR_IDR       = 2,
    PRO_REPLY_ERR_FUNC      = 3,
    PRO_REPLY_ERR_ORDER     = 4,
    PRO_REPLY_ERR_SOCKET    = 5,
    PRO_REPLY_ERR_DATA      = 6,
    PRO_REPLY_ERR_CONNECT   = 7
};

enum pro_order_type
{
    PRO_ORDER_STATE_GET       = 1,
    PRO_ORDER_RESOURCES_GET   = 2
};

struct slave_port_type
{
    uint8_t id;         //扩展机地址
    uint8_t idr;        //扩展机地址取反
    uint8_t sum;        //校验码
    uint8_t func;       //功能码
    uint8_t len;       //数据长度
    uint8_t *data;    //数据区
};
typedef struct slave_port_type *ptr_slave_port_t;
typedef struct slave_port_type g_slave_port_send;

/*port_mng.h*/


#ifndef ARRAY_ELEMENTS
#define ARRAY_ELEMENTS(M) (sizeof(M) / sizeof(M[0]))
#endif
/* 主机和扩展机端口描述 */
#ifndef OBJ_TYPE
typedef unsigned int OBJ_TYPE;
#endif

typedef enum
{
    OBJ_TYPE_NONE = 0,
    OBJ_TYPE_DI,/*数字量输入*/
    OBJ_TYPE_AI,/*模拟量输入*/
    OBJ_TYPE_DO,/*数字量输出*/
    OBJ_TYPE_AO,/*模拟量输出*/
    OBJ_TYPE_RS485,
    OBJ_TYPE_RS232,
    OBJ_TYPE_COMBO, /*多功能接口,同一个端口同时支持232和485*/
    OBJ_TYPE_MAX
}OBJ_TYPE_E;

#define OCAP_CRT          0x80000000                  //端口可创建属性，目前对DI\AI\DO赋予此属性
#define OCAP_NON          0x00000000                  //端口空属性，目前对232\485\COMBO赋予此属性
#define OCAP_NEED_CRT(portCap)	(((portCap) & OCAP_CRT) == OCAP_CRT)


#define DEV_TYPE_M100_DI_PORT_DESC          {USR_M100, (OBJ_TYPE_DI),     OCAP_CRT, 2}
#define DEV_TYPE_M100_AI_PORT_DESC          {USR_M100, (OBJ_TYPE_AI),     OCAP_CRT, 2}
#define DEV_TYPE_M100_DO_PORT_DESC          {USR_M100, (OBJ_TYPE_DO),     OCAP_CRT, 2}
#define DEV_TYPE_M100_COMBO_PORT_DESC       {USR_M100, (OBJ_TYPE_COMBO),  OCAP_NON, 1}
#define DEV_TYPE_M100_RS485_PORT_DESC       {USR_M100, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO4040_DI_PORT_DESC        {USR_IO4040, (OBJ_TYPE_DI),     OCAP_CRT, 4}
#define DEV_TYPE_IO4040_DO_PORT_DESC        {USR_IO4040, (OBJ_TYPE_DO),     OCAP_CRT, 4}
#define DEV_TYPE_IO4040_RS485_PORT_DESC     {USR_IO4040, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO0440_AI_PORT_DESC        {USR_IO0440, (OBJ_TYPE_AI),     OCAP_CRT, 4}
#define DEV_TYPE_IO0440_DO_PORT_DESC        {USR_IO0440, (OBJ_TYPE_DO),     OCAP_CRT, 4}
#define DEV_TYPE_IO0440_RS485_PORT_DESC     {USR_IO0440, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO0080_DO_PORT_DESC        {USR_IO0080, (OBJ_TYPE_DO),     OCAP_CRT, 8}
#define DEV_TYPE_IO0080_RS485_PORT_DESC     {USR_IO0080, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO8000_DI_PORT_DESC        {USR_IO8000, (OBJ_TYPE_DI),     OCAP_CRT, 8}
#define DEV_TYPE_IO8000_RS485_PORT_DESC     {USR_IO8000, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO0800_AI_PORT_DESC        {USR_IO0800, (OBJ_TYPE_AI),     OCAP_CRT, 8}
#define DEV_TYPE_IO0800_RS485_PORT_DESC     {USR_IO0800, (OBJ_TYPE_RS485),  OCAP_NON, 1}

#define DEV_TYPE_IO0404_AI_PORT_DESC        {USR_IO0404, (OBJ_TYPE_AI),     OCAP_CRT, 4}
#define DEV_TYPE_IO0404_AO_PORT_DESC        {USR_IO0404, (OBJ_TYPE_AO),     OCAP_CRT, 4}
#define DEV_TYPE_IO0404_RS485_PORT_DESC     {USR_IO0404, (OBJ_TYPE_RS485),  OCAP_NON, 1}

typedef struct port_des_info
{
    DEV_TYPE uiDevType;             //扩展机型号
    OBJ_TYPE uiPortType;            //端口型号
    uint32_t uiPortTypeClass;    //端口属性
    uint32_t uiPortNum;          //端口数量
}PORT_DES_INFO_T;


/*port_mng.c*/
struct port_des_info g_dev_port_template[] = {
    DEV_TYPE_M100_DI_PORT_DESC,
    DEV_TYPE_M100_AI_PORT_DESC,
    DEV_TYPE_M100_DO_PORT_DESC,
    DEV_TYPE_M100_COMBO_PORT_DESC,
    DEV_TYPE_M100_RS485_PORT_DESC,
    DEV_TYPE_IO4040_DI_PORT_DESC,
    DEV_TYPE_IO4040_DO_PORT_DESC,
    DEV_TYPE_IO4040_RS485_PORT_DESC,
    DEV_TYPE_IO0440_AI_PORT_DESC,
    DEV_TYPE_IO0440_DO_PORT_DESC,
    DEV_TYPE_IO0440_RS485_PORT_DESC,
    DEV_TYPE_IO0080_DO_PORT_DESC,
    DEV_TYPE_IO0080_RS485_PORT_DESC,
    DEV_TYPE_IO8000_DI_PORT_DESC,
    DEV_TYPE_IO8000_RS485_PORT_DESC,
    DEV_TYPE_IO0800_AI_PORT_DESC,
    DEV_TYPE_IO0800_RS485_PORT_DESC,
    DEV_TYPE_IO0404_AI_PORT_DESC,
    DEV_TYPE_IO0404_AO_PORT_DESC,
    DEV_TYPE_IO0404_RS485_PORT_DESC
};



/*slot_mng.h*/
typedef enum
{
	SLOT_STATUS_NULL = 0,
	SLOT_STATUS_OK,
	SLOT_STATUS_OFFLINE,	
	SLOT_STATUS_ERR,
	SLOT_STATUS_MAX
}SLOT_STATUS_E;

typedef struct slot_mem_info
{
	DEV_TYPE uiRegType;     //预配置扩展机型号
	DEV_TYPE uiRunType;     //当前扩展机型号
	uint32_t uiStatus;   //当前槽位状态
    uint32_t  ackTick;    //接收到数据的时间
	uint8_t ext_ver[24];
}SLOT_MEM_INFO_T;


#if 0
const char *apcSlotStaut[SLOT_STATUS_MAX] = {\
    "NULL",\
    "OK",\
    "OFFLINE",\
    "ERR",\
};
#endif
//CRC校验模板
const uint16_t CRCTalbeAbs[] = {
    0x0000,
    0xCC01,
    0xD801,
    0x1400,
    0xF001,
    0x3C00,
    0x2800,
    0xE401,
    0xA001,
    0x6C00,
    0x7800,
    0xB401,
    0x5000,
    0x9C01,
    0x8801,
    0x4400,
};



/*io_modbus_slave.h*/
//扩展机相关
#define EXT_DI_DO_AI_SWITCH_START   ((uint16_t)0x0000)
#define EXT_VER_REGISTER_ADDR       ((uint16_t)0x0050)
#define EXT_VER_REGISTER_LEN        ((uint16_t)0x000A)
#define EXT_DI_SWITCH_START         EXT_DI_DO_AI_SWITCH_START
#define EXT_DO_SWITCH_START         EXT_DI_DO_AI_SWITCH_START
#define EXT_AI_SWITCH_START         EXT_DI_DO_AI_SWITCH_START
#define EXT_DI_FILTER_TIME          ((uint16_t)0x0030)
#define EXT_RESET_ADDR              ((uint16_t)0x0010)
#define EXT_WORK_MODE               ((uint16_t)0x0040)
#define EXT_AO_MODE                 ((uint16_t)0x0190)
#define EXT_AO_SWITCH_START         ((uint16_t)0x0000)
/*slot_mng.c*/
uint8_t led_sta_flag[MAX_SLOT_NUM] = {0};
//struct slot_mem_info g_pstSlotInfo[MAX_SLOT_NUM] = {0};
struct slot_mem_info *g_pstSlotInfo = RT_NULL;


volatile SemaphoreHandle_t logprint_serial_in_userd_Semaphore;


/*rt链表*/
/**
 * Double List structure
 */
struct rt_list_node
{
    struct rt_list_node *next;                          /**< point to next node. */
    struct rt_list_node *prev;                          /**< point to prev node. */
};
typedef struct rt_list_node rt_list_t;                  /**< Type for lists. */

/**
 * rt_container_of - return the start address of struct type, while ptr is the
 * member of struct type.
 */
#define rt_container_of(ptr, type, member) \
    ((type *)((char *)(ptr) - (unsigned long)(&((type *)0)->member)))

/**
 * @brief get the struct for this entry
 * @param node the entry point
 * @param type the type of structure
 * @param member the name of list in structure
 */
#define rt_list_entry(node, type, member) \
    rt_container_of(node, type, member)

/**
 * rt_list_for_each - iterate over a list
 * @pos:    the rt_list_t * to use as a loop cursor.
 * @head:   the head for your list.
 */
#define rt_list_for_each(pos, head) \
    for (pos = (head)->next; pos != (head); pos = pos->next)

 /**
 * @brief initialize a list
 *
 * @param l 操作的链表
 @param n 插入的新节点
 */   
     rt_inline void rt_list_insert_before(rt_list_t *l,rt_list_t *n)
     {
        l->prev->next=n;
        n->prev=l->prev;
        l->prev=n;
        n->next=l;

     }
/**
 * @brief initialize a list
 *
 * @param l list to be initialized
 */
rt_inline void rt_list_init(rt_list_t *l)
{
    l->next = l->prev = l;
}

/*port_mng.h*/
#define MAX_PORT_NAME_LEN    6    //  DO/18
#define MAX_IFINDEX_NAME_LEN 7    //  DO/1/8


/* LOGIC PORT通用解析宏函数*/
#define DEV_LPORT_2_TYPE(lport)             ((lport) >> 8)
#define DEV_LPORT_2_XXID(lport)             ((lport) & 0XFF)
#define DEV_LPORT_ASSEMBLE(type, xxid)      (((type) << 8) | (xxid))    //  DO/18

typedef union port_value_info
{
    float       fValue;   /*AI当前值*/
    uint8_t uiValue;  /*DI/DO当前值*/
}PORT_VALUE_U;

typedef struct port_mem_info
{
    uint16_t usLport;                //DO/18
    uint32_t uiIfIndex;              //1/DO/8
    union port_value_info port_value;
    rt_list_t stPortMemList;
}PORT_MEM_T;

typedef struct port_glb_info
{
    uint8_t aPortNum[OBJ_TYPE_MAX];
    rt_list_t stPortMemList;
}PORT_GLB_T;

/* IFINDEX通用解析宏函数 */
#define DEV_IFINDEX_2_SLOT(ifIndex)             (((ifIndex) >> 24) & 0x0F) 
#define DEV_IFINDEX_2_TYPE(ifIndex)             (((ifIndex) >> 16) & 0x3F)
#define DEV_IFINDEX_2_XXID(ifIndex)             ((ifIndex) & 0x0000FFFF)
#define DEV_IFINDEX_ASSEMBLE(slot, type, xxid)  (((slot)<<24) | ((type)<<16) | (xxid))  //  1/DO/8

#if 1
#define DEBUG_BUF_SIZE 200
char gv_debug_buff[DEBUG_BUF_SIZE];

void debug_to_rs485_1( char *str_result) 
{

  long buf_length=0;
  RS485.noReceive();

  RS485.beginTransmission();
  buf_length=strlen(str_result);

  for (int i = 0; i < buf_length; i++) 
  {
    
    RS485.write(str_result[i]);
    
   
  }
  RS485.endTransmission();
  //Serial.println("response_newcommandresult  end");
}


#endif
/*port_mng.c*/
//struct port_glb_info *g_pstPort = NULL;
// struct port_glb_info g_pstPort = {0};

// int port_mng_init()
// {
//     g_pstPort = (g_pstPort)malloc(sizeof(struct port_glb_info));
// 	if(!g_pstPort)
// 	{
//         // LOG_STD(LOG_ERR, LOG_TAG, "g_pstPort malloc err!\n");
// 		return RT_ERROR;
// 	}
// 	memset(g_pstPort, 0, sizeof(struct port_glb_info));
// 	rt_list_init(&g_pstPort->stPortMemList);
// 	return RT_EOK;
// }
/*port_mng.c*/
struct port_glb_info *g_pstPort = NULL;

 int port_mng_init()
 {
     g_pstPort = (port_glb_info*)malloc(sizeof(struct port_glb_info));
 	if(!g_pstPort)
 	{
//         // LOG_STD(LOG_ERR, LOG_TAG, "g_pstPort malloc %d err!\n",sizeof(struct port_glb_info));

        //memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
        //stncpy(gv_debug_buff," [line 420] [LOG_ERR][port_mng_init]  g_pstPort malloc err! \n",DEBUG_BUF_SIZE,);
        debug_to_rs485_1(" [line 420] [LOG_ERR][port_mng_init]  g_pstPort malloc err! \n");
 		return RT_ERROR;
 	}
 	memset(g_pstPort, 0, sizeof(struct port_glb_info));
 	rt_list_init(&g_pstPort->stPortMemList);
 	return RT_EOK;
 }
/*protocol_extender.c*/
uint32_t  do_set_ack_tick[MAX_IFINDEX_NAME_LEN] = {0};
uint32_t  ao_set_ack_tick[MAX_SLOT_NUM] = {0};

uint8_t gv_recv_flag = 0;

#if 0
#define RELOAD_DOG_PIN 3
static void reload_dog_in_usrlib(void) 
{

    //return;
  pinMode(RELOAD_DOG_PIN, OUTPUT);
  digitalWrite(RELOAD_DOG_PIN, 0);
  delay(100);
  digitalWrite(RELOAD_DOG_PIN, 1);
  delay(250);
  digitalWrite(RELOAD_DOG_PIN, 0);
  delay(100);
}
#endif
/**
 * @brief hal_crc16     crc校验函数
 * @param pchMsg        char * 类型的指针,需要被校验的数据
 * @param wDataLen      数据长度
 * @return
 */
uint16_t crc16(char *pchMsg, uint16_t wDataLen)
{
  uint16_t wCRC = 0xFFFF;
  uint16_t i;
  uint8_t chChar;
  for (i = 0; i < wDataLen; i++)
  {
    chChar = *pchMsg++;
    wCRC = CRCTalbeAbs[(chChar ^ wCRC) & 15] ^ (wCRC >> 4);
    wCRC = CRCTalbeAbs[((chChar >> 4) ^ wCRC) & 15] ^ (wCRC >> 4);
  }
  return (wCRC << 8) + (wCRC >> 8);
}

char log_flag = 0;
/*串口2，主机通过485和扩展机交互*/
void uart3_send(unsigned char *sendstr, int send_data_len)
{
		pinMode(TX2DEPIN, OUTPUT);
		digitalWrite(TX2DEPIN, HIGH);
		Serial2.write(sendstr, send_data_len);
		Serial2.flush();
		pinMode(TX2DEPIN, OUTPUT);
		digitalWrite(TX2DEPIN, LOW);
}
/*串口0，下载口TX*/
void uart7_send(unsigned char *sendstr, int send_data_len)
{
    Serial.write(sendstr, send_data_len);
}

char di_status[MAX_SLOT_NUM][8] = {-1};
float ai_status[MAX_SLOT_NUM][8] = {0};
char do_status_set[MAX_SLOT_NUM] = {0};

void modbus_analysis(uint8_t *mbdata, uint8_t mblen, uint8_t *recv,  uint8_t flag)
{
    float *val;
    uint16_t crc;
    uint8_t ai_num = 0;
    uint8_t ao_num = 0;
    float ai_val[8] = {0};
    float ao_val[8] = {0};
    uint8_t p_val[4] = {0};
    uint8_t do_val[8] = {0};
    uint8_t di_val[8] = {0};
    rt_list_t *pstList = RT_NULL;
    PORT_MEM_T *pstPortMem = RT_NULL;


            //  memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
            //  snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 504] modbus_analysis mbdata[0]=%d  mbdata[1]=%x flag=%d \n",mbdata[0], mbdata[1],flag);
            //  debug_to_rs485_1(gv_debug_buff);

    if (mbdata[0] < MIN_SLOT_NUM || mbdata[0] >= MAX_SLOT_NUM)
    {

        debug_to_rs485_1("[line 532][modbus_analysis][LOG_WARNING] modbus addr err  \n");

        return;
    }
    crc = crc16((char *)mbdata, mblen); //计算CRC校验值//这个函数有问题，后面更换
    if (crc != 0)
    {
  
            debug_to_rs485_1("[line 540] [modbus_analysis] [LOG_WARNING]crc err \n");
        return;
    }

    if (mbdata[1] & 0x80)
    {
        return;
    }

    if(flag)
    {
        if(recv)
        {
            memcpy(recv, mbdata, mblen);
        }
        else
        {
    


        debug_to_rs485_1("modbus recv buf err\n");
        }

    }
    else
    {
        switch (mbdata[1]) //功能码
        {
            case IO_READ_COIL: // 0x01
            {   
                for(int i = 0; i < 8; i++)
                {
                    do_val[i] = (mbdata[3] >> i) & 0x01;
                }
                do_set_ack_tick[mbdata[0]] = millis();
                break;
            }
            case IO_READ_INPUT: // 0x02
            {
                for(int i = 0; i < 8; i++)
                {
                    di_val[i] = (mbdata[3] >> i) & 0x01;
                    di_status[mbdata[0]][i] = di_val[i];

                     // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
                     // snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 585] di_status[ %d ][%d]=%d   \n",mbdata[0],i, di_status[mbdata[0]][i]);
                     // debug_to_rs485_1(gv_debug_buff);
                }
                break;
            }
            case IO_READ_HOLDING_REGISTER: // 0x03
            {
                   #if 1 
                    if(mbdata[2] == 0x14)
                    {
                        memcpy(g_pstSlotInfo[mbdata[0]].ext_ver, &mbdata[3], mbdata[2]);
                    }
                    else
                    {
                        ao_num = 4;

                        for(int i = 0; i < ao_num; i++)
                        {
                            
                            for(int j = 0; j < 4; j++)
                            {
                                p_val[j] = mbdata[3 + 4 * i + 3 - j];
                            }  
                            val = (float*)p_val;
                            ao_val[i] = *val;
                        }
                        ao_set_ack_tick[mbdata[0]] = millis();
                    }
                 #endif
            }
                break;
            
            case IO_READ_INPUT_REGISTER: // 0x04
            {
            #if 1
            //char debug_buff_ai_val[DEBUG_BUF_SIZE];
                if(g_pstSlotInfo[mbdata[0]].uiRunType == USR_IO0800)
                {
                    ai_num = 8;
                }
                else 
                {
                    ai_num = 4;
                }
                for(int i = 0; i < ai_num; i++)
                {
                    
                    for(int j = 0; j < 4; j++)
                    {
                        p_val[j] = mbdata[3 + 4 * i + 3 - j];
                    }  
                    val = (float*)p_val;
                    ai_val[i] = *val;
                    ai_status[mbdata[0]][i]=ai_val[i];
                     
                }



                #endif
             }
                
                break;
           
            default: ///***功能码不支持***///
            {
                return;
            }
        }

        //eg118执行了这个list的循环就会死机，需要探查原因
        #if 1
        int listloopcount=0;
         if ( &g_pstPort->stPortMemList==RT_NULL )
         {

            // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
             //strncpy(gv_debug_buff," [line 689] [modbus_analysis]   &g_pstPort->stPortMemList==RT_NULL \n",DEBUG_BUF_SIZE);
             debug_to_rs485_1(" [line 689] [modbus_analysis]   &g_pstPort->stPortMemList==RT_NULL \n");
         }

        rt_list_for_each(pstList, &g_pstPort->stPortMemList) //遍历所有端口
        {
            listloopcount++;
            pstPortMem = rt_list_entry(pstList, PORT_MEM_T, stPortMemList);




            if(DEV_IFINDEX_2_SLOT(pstPortMem->uiIfIndex) == mbdata[0])
            {

            //  memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
            //  snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 678] modbus_analysis   DEV_IFINDEX_2_TYPE(pstPortMem->uiIfIndex)=%d  \n",DEV_IFINDEX_2_TYPE(pstPortMem->uiIfIndex));
            //  debug_to_rs485_1(gv_debug_buff);


                switch(DEV_IFINDEX_2_TYPE(pstPortMem->uiIfIndex))
                {
                    case OBJ_TYPE_DI:
                        if(mbdata[1] == IO_READ_INPUT)
                        {
                            pstPortMem->port_value.uiValue = di_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1];
                        }
                        break;
                    case OBJ_TYPE_DO:
                        if(mbdata[1] == IO_READ_COIL)
                        {
                            pstPortMem->port_value.uiValue = do_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1];
                        }   
                        break;
                    case OBJ_TYPE_AI:
                        if(mbdata[1] == IO_READ_INPUT_REGISTER)
                        {
                            if(ai_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1] <= 1000)  //cyl 2023 0423 此处为了跟主机AI同步，若主机修改，此处需要同步修改
                            {
                                pstPortMem->port_value.fValue = 0;
                                //     if (xSemaphoreTake(logprint_serial_in_userd_Semaphore, 0) == pdTRUE)
                                //     {
                                            
                                //         memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
                                //         snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 723]    fValue = %f \n",pstPortMem->port_value.fValue);
                                //         debug_to_rs485_1(gv_debug_buff);
                                //         delay(500);
                                //     }
                                // xSemaphoreGive(logprint_serial_in_userd_Semaphore);
                            }
                            else if(ai_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1] > 25000)
                            {
                                pstPortMem->port_value.fValue = 25000;
                                if (xSemaphoreTake(logprint_serial_in_userd_Semaphore, 0) == pdTRUE)
                                {
                                    
                                    // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
                                    // snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 736]    fValue = %f \n",pstPortMem->port_value.fValue);
                                    // debug_to_rs485_1(gv_debug_buff);
                                    // delay(500);
                                }
                                xSemaphoreGive(logprint_serial_in_userd_Semaphore);
      
                            }
                            else
                            {
                                pstPortMem->port_value.fValue  = ai_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1]; 
                                    if (xSemaphoreTake(logprint_serial_in_userd_Semaphore, 0) == pdTRUE)
                                        {
                                            
                                           // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
                                           // snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 750]    fValue = %fmA\n",pstPortMem->port_value.fValue/1000);
                                           // debug_to_rs485_1(gv_debug_buff);
                                            delay(500);
                                        }
                                xSemaphoreGive(logprint_serial_in_userd_Semaphore);
                            }
                        }    
                        break;
                    case OBJ_TYPE_AO:
                        if(mbdata[1] == IO_READ_HOLDING_REGISTER)
                        {
                            pstPortMem->port_value.fValue = ao_val[DEV_IFINDEX_2_XXID(pstPortMem->uiIfIndex) - 1];
                        }    
                        break;
                    default:
                        break;
                }//switch
            }//if(DEV_IFINDEX_2_SLOT(pstPortMem->uiIfIndex) == mbdata[0])
           
        }//rt_list_for_each
         #endif
        //    memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
        //     snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 702] [modbus_analysis]   end rt_list_for_each  listloopcount=%d   \n",listloopcount);
        //     debug_to_rs485_1(gv_debug_buff);





            // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
            // snprintf(gv_debug_buff,DEBUG_BUF_SIZE," [line 572] modbus_analysis   end rt_list_for_each  mbdata[0]=%d  mbdata[1]=%x flag=%d \n",mbdata[0], mbdata[1],flag);
            // debug_to_rs485_1(gv_debug_buff);
            return;
    }

    return; /// num为返回MODBUS协议的数据格式
}   //end  modbus_analysis

void modbus_unpack(uint8_t *data, uint8_t len, uint8_t *recv,  uint8_t flag)
{
   // uint8_t start = 0;
    unsigned short target_crc=0;
    unsigned short new_crc=0;


    if (data[0] < MIN_SLOT_NUM || data[0] >= MAX_SLOT_NUM)
        return;

    for(int i = 0; i < len - 1; i++)
    {
        if(data[i] == data[0])
        {
            
        // memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
        // snprintf(gv_debug_buff,DEBUG_BUF_SIZE," line 722 [modbus_unpack] switch %x \n", (data[i + 1]) );
        //return;

        //debug_to_rs485_1(gv_debug_buff);
            switch (data[i + 1])
            {
                case IO_READ_COIL:
                case IO_READ_INPUT:
                case IO_READ_HOLDING_REGISTER:
                case IO_READ_INPUT_REGISTER:

                //eg118工程代码与m100的原始代码调整这里的高8位与低8位。
                target_crc=(data[i + data[i + 2] + 3] << 8) | data[i + data[i + 2] + 4];//data[i + data[i + 2] + 4]
                new_crc=crc16((char *)&data[i], data[i + 2] + 3);

                    if (target_crc == new_crc )
                        {
                            modbus_analysis(&data[i], data[i + 2] + 5, recv, flag);
                            i = i + data[i + 2] + 4;
                        }
                    else
                        {

                            debug_to_rs485_1(" line 747 crc16  error \n");

                        }
                    break;
                case IO_WRITE_COIL:
                case IO_WRITE_REGISTER:
                case IO_WRITE_COIL_ANY:
                case IO_WRITE_REG_ANY:
                    if(((data[i + 7] << 8) | data[i + 6]) == crc16((char *)&data[i], 6))
                        {
                            modbus_analysis(&data[i], 8, recv, flag);
                            i = i + 7;
                        }

                    break;
                default:
                    break;
            }
        }
    }

    return;
}   //modbus_unpack


uint8_t *modbus_package_do_only(uint8_t addr, uint8_t order, uint16_t register_addr, uint16_t len, uint16_t *data, uint8_t *buf)
{
    uint16_t crc;
    uint16_t data_len = len;

    uint8_t *buf_p = buf;

    *buf_p++ = addr;
    *buf_p++ = order;
    *buf_p++ = (register_addr >> 8) & 0xff;
    *buf_p++ = register_addr & 0xff;
    switch (order)
    {
    case IO_READ_COIL:
    case IO_READ_INPUT:
    case IO_READ_HOLDING_REGISTER:
    case IO_READ_INPUT_REGISTER:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        break;
    case IO_WRITE_COIL:
    case IO_WRITE_REGISTER:
        *buf_p++ = (*data >> 8) & 0xff;
        *buf_p++ = *data & 0xff;
        break;
    case IO_WRITE_COIL_ANY:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        *buf_p++ = (data_len - 1) / 8 + 1;
        for (uint16_t i = 0; i < ((data_len - 1) / 8 + 1); i++)
        {
            *buf_p++ = *data & 0xff;
            data++;
        }
        break;
    case IO_WRITE_REG_ANY:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        *buf_p++ = 2 * data_len;
        for (uint16_t i = 0; i < data_len; i++)
        {
            *buf_p++ = (*data >> 8) & 0xff;
            *buf_p++ = *data & 0xff;
            data++;
        }
        break;
    default:
        break;
    }

    crc = crc16((char *)buf, (buf_p - buf)); //这个函数有问题，后面更换

    
    *buf_p++ = crc >> 8;
    *buf_p++ = (crc & 0xFF);

    return buf_p;
}// end modbus_package_do_only

uint8_t *modbus_package(uint8_t addr, uint8_t order, uint16_t register_addr, uint16_t len, uint16_t *data, uint8_t *buf)
{
    uint16_t crc;
    uint16_t data_len = len;

    uint8_t *buf_p = buf;
    uint8_t register_addr_HIGH8BIT = (register_addr >> 8) & 0xff;
    uint8_t register_addr_LOW8BIT = register_addr & 0xff;
    

    *buf_p++ = addr;
    *buf_p++ = order;
    *buf_p++ = register_addr_HIGH8BIT;
    *buf_p++ = register_addr_LOW8BIT;
    switch (order)
    {
    case IO_READ_COIL:
    case IO_READ_INPUT:
    case IO_READ_HOLDING_REGISTER:
    case IO_READ_INPUT_REGISTER:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        break;
    case IO_WRITE_COIL:
    case IO_WRITE_REGISTER:
        *buf_p++ = (*data >> 8) & 0xff;
        *buf_p++ = *data & 0xff;
        break;
    case IO_WRITE_COIL_ANY:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        *buf_p++ = (data_len - 1) / 8 + 1;
        for (uint16_t i = 0; i < ((data_len - 1) / 8 + 1); i++)
        {
            *buf_p++ = *data & 0xff;
            data++;
        }
        break;
    case IO_WRITE_REG_ANY:
        *buf_p++ = (len >> 8) & 0xff;
        *buf_p++ = len & 0xff;
        *buf_p++ = 2 * data_len;
        for (uint16_t i = 0; i < data_len; i++)
        {
            *buf_p++ = (*data >> 8) & 0xff;
            *buf_p++ = *data & 0xff;
            data++;
        }
        break;
    default:
        break;
    }

    crc = crc16((char *)buf, (buf_p - buf)); //这个函数有问题，后面更换
    *buf_p++ = crc >> 8;
    *buf_p++ = (crc & 0xFF);
    
    return buf_p;
}// end modbus_package


int protocol_extender_package(slave_port_type *info)
{
    uint16_t data = 0;
    for(uint16_t i = 0; i < info->len; i++)//节省RAM不再采用中间变量中转数据，发送数据的数组要比数据大5字节
    {
        info->data[info->len - 1 - i + PROTOCOL_ADD_LEN] = info->data[info->len - 1 - i];
    }

    info->data[0] = info->id;
    info->data[1] = info->idr;
    info->data[3] = info->func;
    info->data[4] = info->len;
 
    for(uint16_t i = 3; i < info->len + PROTOCOL_ADD_LEN; i++)
    {
        data+= info->data[i];

    }
    info->data[2] = data % 256;

    return info->len + PROTOCOL_ADD_LEN;
}

slave_port_type* protocol_extender_analysis(uint8_t *buf, uint16_t len)
{
    uint32_t checksum = 0;
    uint8_t code = 0;
    uint8_t idr = 0;

    slave_port_type *ptr_slave_port_extender_analysis=0;
    ptr_slave_port_extender_analysis=(slave_port_type*)malloc( sizeof (slave_port_type));
    
    if ( RT_NULL==ptr_slave_port_extender_analysis )
    {
        debug_to_rs485_1("[line 989][protocol_extender_analysis][LOG_ERR] malloc failed!\n");
        return RT_NULL;
    }
    memset(ptr_slave_port_extender_analysis, 0, (sizeof(slave_port_type)));

    code = PRO_REPLY_SUCCESS;
    do
    {
            // memset(gv_debug_buff,0,100);
            // snprintf(gv_debug_buff,100," [line 1078] [protocol_extender_analysis]-> len =%d \n", len);
            // debug_to_rs485_1(gv_debug_buff);
        for(uint16_t i = 3; i < len; i++)
        {
            checksum += buf[i];
            
            // memset(gv_debug_buff,0,100);
            // snprintf(gv_debug_buff,100," [protocol_extender_analysis] buf[%d] =0x%x \n", i,buf[i]);
            // debug_to_rs485_1(gv_debug_buff);

        }
        checksum = checksum % 256;

        if(checksum != buf[2])
        {
            code = PRO_REPLY_ERR_SUM;

            break;
        }

        idr = ~buf[0];
        if(buf[1] != idr)
        {
            code = PRO_REPLY_ERR_IDR;
            //LOG_STD(LOG_WARNING, LOG_TAG, "extender analysis idr err!\n");

            debug_to_rs485_1("[line 761] [LOG_WARNING] extender analysis idr err!\n");
            break;
        }

        if((buf[3] & 0xF0) != PRO_ORDER_REPLY_CORRECT)
        {
            code = PRO_REPLY_ERR_FUNC;
            debug_to_rs485_1("[line 1032] [LOG_WARNING] extender analysis func err!\n");
            break;
        }

        ptr_slave_port_extender_analysis->id = buf[0];
        ptr_slave_port_extender_analysis->idr = buf[1];
        ptr_slave_port_extender_analysis->sum = buf[2];
        ptr_slave_port_extender_analysis->func = buf[3];
        ptr_slave_port_extender_analysis->len = buf[4];

        if ((ptr_slave_port_extender_analysis->data = (uint8_t *)malloc(ptr_slave_port_extender_analysis->len + 1)) == RT_NULL)
        {
            debug_to_rs485_1("[line 1047][LOG_WARNING] port_t->buf malloc failed!\n");
            free(ptr_slave_port_extender_analysis);
            return RT_NULL;
        }

        memset(ptr_slave_port_extender_analysis->data, 0, ptr_slave_port_extender_analysis->len + 1);
        memcpy(ptr_slave_port_extender_analysis->data, &buf[5], ptr_slave_port_extender_analysis->len);

    } while (0);
    
    if(code != PRO_REPLY_SUCCESS)
    {
        free(ptr_slave_port_extender_analysis);
        ptr_slave_port_extender_analysis = RT_NULL;
    }
    return ptr_slave_port_extender_analysis;
}//protocol_extender_analysis



int protocol_extender_send(uint16_t len, uint8_t slot_num, uint8_t func, uint8_t *data, uint32_t timeout)
{
  //  uint32_t evt;
#if 1
    slave_port_type *ptr_send_port = (slave_port_type*)malloc(sizeof(slave_port_type));

    if (ptr_send_port == 0)
    {
       // Serial.println("send_port_t malloc err!");
            debug_to_rs485_1("usr_io_task line 802 send_port_t malloc err!\n");

        return -1;
    }
    memset(ptr_send_port, 0, sizeof(slave_port_type));

    ptr_send_port->id = slot_num;
    ptr_send_port->idr = ~slot_num;
    ptr_send_port->func = func;
    ptr_send_port->len = len;
    ptr_send_port->data = data;

    int send_len = protocol_extender_package(ptr_send_port);

    // rt_event_recv(&recv_evt, 1, RT_EVENT_FLAG_AND | RT_EVENT_FLAG_CLEAR, timeout, &evt);


    uart3_send(ptr_send_port->data, send_len);
     
    //Serial.println("send_port_t malloc err!");
    free(ptr_send_port);
    ptr_send_port = 0;
#else
   // g_slave_port_send = {0};
    memset(&g_slave_port_send, 0, sizeof( slave_port_type ) );

    g_slave_port_send.id = slot_num;
    g_slave_port_send.idr = ~slot_num;
    g_slave_port_send.func = func;
    g_slave_port_send.len = len;
    g_slave_port_send.data = data;

    int send_len = protocol_extender_package(&g_slave_port_send);
    delay(timeout);
    uart3_send(g_slave_port_send.data, send_len);
#endif
    return 0;
}

/*slot_mng.c*/
void ext_status_led_set(uint8_t slot_num, uint8_t led_set)
{

    uint8_t *send_buf = NULL;
    uint8_t send_data[16] = {0};

    uint16_t u16led_level[2];
    //return;
    if (led_set == LED_ON)
    {
        u16led_level[0]=0x01;
        send_buf = modbus_package(slot_num, IO_WRITE_REGISTER, EXT_WORK_MODE, 0, u16led_level, send_data);
    }
    else
    {
         u16led_level[0]=0x00;
        send_buf = modbus_package(slot_num, IO_WRITE_REGISTER, EXT_WORK_MODE, 0, u16led_level, send_data);
    }

    protocol_extender_send((send_buf - send_data), slot_num, PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, EXT_POLL_TIMEOUT);  //修改扩展机work灯状态
}



#define USR_M100_NAME   "USR-EG118"
#define USR_IO4040_NAME "USR-IO4040"
#define USR_IO0440_NAME "USR-IO0440"
#define USR_IO0080_NAME "USR-IO0080"
#define USR_IO8000_NAME "USR-IO8000"
#define USR_IO0800_NAME "USR-IO0800"
#define USR_IO0404_NAME "USR-IO0404"

#define MAX_DEV_NAME_LEN 11

int dev_get_name_by_type(DEV_TYPE uiDevType, char *pcDevName)
{
    switch(uiDevType)
    {
        case USR_M100:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_M100_NAME);
            break;
        case USR_IO4040:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO4040_NAME);
            break;
        case USR_IO0440:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO0440_NAME);
            break;
        case USR_IO0080:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO0080_NAME);
            break;
        case USR_IO8000:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO8000_NAME);
            break;
        case USR_IO0800:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO0800_NAME);
            break;
        case USR_IO0404:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, USR_IO0404_NAME);
            break;
        case USR_DEV_NULL:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, "NONE");
            break;		
        default:
            snprintf(pcDevName, MAX_DEV_NAME_LEN, "UNKNOW_%x",uiDevType);
            break;
    }
    return RT_EOK;
}

DEV_TYPE dev_get_type_by_extender(DEV_TYPE uiDevType)
{
    switch(uiDevType)
    {
        case USR_IO4040_EXT:
            return USR_IO4040;
        case USR_IO0440_EXT:
            return USR_IO0440;
        case USR_IO0080_EXT:
            return USR_IO0080;
        case USR_IO8000_EXT:
            return USR_IO8000;
        case USR_IO0800_EXT:
            return USR_IO0800;
        case USR_IO0404_EXT:
            return USR_IO0404;
        case USR_DEV_NULL:
            return USR_DEV_NULL;
        default:
            return USR_DEV_UNKNOW;
    }
}

struct port_mem_info *port_mng_mem_new(void)
{
    struct port_mem_info *pstPort = (port_mem_info*)malloc(sizeof(struct port_mem_info));
    if(!pstPort)
    {
        debug_to_rs485_1("[line 1203] [LOG_ERR]pstPort malloc err! \n");

        return RT_NULL;
    }
    memset(pstPort, 0, sizeof(struct port_mem_info));
    return pstPort;
}

int port_intf_create(uint32_t *puiIfIndex)
{
    rt_list_t *pstList = NULL;
    OBJ_TYPE uiPortType = 0;
    uint8_t aPortCnt[OBJ_TYPE_MAX] = {0};
    struct port_mem_info *pstPortMemNew = NULL;
    struct port_mem_info *pstPortMem = NULL;

    if(NULL == puiIfIndex)
    {
        debug_to_rs485_1(" line 1225  puiIfIndex is NULL! \n");
        return RT_ERROR;
    }

    if(g_pstPort == RT_NULL)
    {
        return RT_ERROR;
    }

    pstPortMemNew = port_mng_mem_new();
    if(NULL == pstPortMemNew)
    {

        debug_to_rs485_1(" [line 1234] [port_intf_create] pstPortMemNew is NULL! \n");
        return RT_ERROR;
    }


    rt_list_for_each(pstList, &g_pstPort->stPortMemList) //遍历所有端口
    {
        pstPortMem = rt_list_entry(pstList, PORT_MEM_T, stPortMemList);
        if(*puiIfIndex == pstPortMem->uiIfIndex)
        {
            /*配置索引与已知索引重复，返回错误*/    
            return RT_ERROR;
        }

        uiPortType = DEV_IFINDEX_2_TYPE(pstPortMem->uiIfIndex);
        if(uiPortType < OBJ_TYPE_MAX)
        {
            aPortCnt[uiPortType]++;
        }
    }
    pstPortMemNew->uiIfIndex = *puiIfIndex;
    uiPortType = DEV_IFINDEX_2_TYPE(*puiIfIndex);
    if(uiPortType < OBJ_TYPE_MAX)
    {
        pstPortMemNew->usLport = DEV_LPORT_ASSEMBLE(uiPortType, aPortCnt[uiPortType] + 1);
    }

             //return RT_ERROR;


    rt_list_insert_before(pstList, &pstPortMemNew->stPortMemList);////往前插入一个节点


            
    return RT_EOK;
}

int port_add(uint8_t ucSlotId, DEV_TYPE uiBodType)
{
    int i;
    uint32_t uiPhyId;
    uint32_t uiInterface;

    if((uiBodType < USR_M100) || (uiBodType >= USR_DEV_UNKNOW))
    {
        debug_to_rs485_1(" line 1283 RT_ERROR  (uiBodType < USR_M100) || (uiBodType >= USR_DEV_UNKNOW) \n");
        return RT_ERROR;
    }

    if(g_pstPort == RT_NULL)
    {

        debug_to_rs485_1(" line 1326  (g_pstPort == RT_NULL) \n");
        return RT_ERROR;
    }

    for(i = 0; i < ARRAY_ELEMENTS(g_dev_port_template); i++)
    {
        
        //这里if0 之后系统就不死机了 看来是这段代码的问题
        if((g_dev_port_template[i].uiDevType == uiBodType) && \
            (OCAP_NEED_CRT(g_dev_port_template[i].uiPortTypeClass)))
        {
            for(uiPhyId = 1; uiPhyId <= g_dev_port_template[i].uiPortNum; uiPhyId++)
            {
               
                uiInterface = DEV_IFINDEX_ASSEMBLE(ucSlotId, g_dev_port_template[i].uiPortType, uiPhyId);
                
                port_intf_create(&uiInterface);
                
                g_pstPort->aPortNum[g_dev_port_template[i].uiPortType]++;
            }			
        }
       
    }// for
   


    return RT_EOK;
}


int slot_reg_dev(uint8_t ucSlot, DEV_TYPE uiDevType)
{
	//char aRegName[MAX_DEV_NAME_LEN] = {0};

    char slot_debug_buff[100];

	if(ucSlot >= MAX_SLOT_NUM)
	{

    debug_to_rs485_1("ucSlot >= MAX_SLOT_NUM\n");
		return RT_ERROR;
	}

	if((uiDevType < USR_M100) || (uiDevType >= USR_DEV_UNKNOW))
	{
        memset(slot_debug_buff,0,100);
        snprintf(slot_debug_buff,100,"[line 1336 ]uiDevType = %x\n",uiDevType);
        debug_to_rs485_1(slot_debug_buff);
    debug_to_rs485_1("[line 1338] [slot_reg_dev] [LOG_ERR] uiDevType < USR_M100) || (uiDevType >= USR_DEV_UNKNOW)\n");
		return RT_ERROR;
	}

	if(ucSlot == FIX_SLOT_NUM)
	{
		g_pstSlotInfo[ucSlot].uiRunType = uiDevType;
		g_pstSlotInfo[ucSlot].uiStatus = SLOT_STATUS_OK;
	}
	else
	{
		g_pstSlotInfo[ucSlot].uiRunType = uiDevType;
		g_pstSlotInfo[ucSlot].uiStatus = SLOT_STATUS_OFFLINE;
	}
	return port_add(ucSlot, uiDevType);	
   
}
#if 0
int slot_show_info()
{
    int i;
    char aRegName[MAX_DEV_NAME_LEN] = {0};
    char aRunName[MAX_DEV_NAME_LEN] = {0};
    memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
    sprintf(gv_debug_buff, " %10s %10s %10s %10s %13s \r\n", "slotid", "regtype","runtype", "status", "ver");
   // sprintf("gv_debug_buff," %10s %10s %10s %10s %13s \r\n", "slotid", "regtype", "runtype", "status", "ver");
    debug_to_rs485_1(gv_debug_buff);

    memset(gv_debug_buff,0,DEBUG_BUF_SIZE);
    snprintf(gv_debug_buff,DEBUG_BUF_SIZE,"---------   ---------  ---------   --------   ------------------\r\n");
    debug_to_rs485_1(gv_debug_buff);
    for(i = FIX_SLOT_NUM; i < MAX_SLOT_NUM; i++)
    {
        dev_get_name_by_type(g_pstSlotInfo[i].uiRegType, aRegName);
        dev_get_name_by_type(g_pstSlotInfo[i].uiRunType, aRunName);	
        memset(gv_debug_buff,0,DEBUG_BUF_SIZE);		
        snprintf(gv_debug_buff,DEBUG_BUF_SIZE,"%10d %10s %10s %10s %21s\r\n", i, aRegName, aRunName, apcSlotStaut[g_pstSlotInfo[i].uiStatus], g_pstSlotInfo[i].ext_ver);
        debug_to_rs485_1(gv_debug_buff);

    }
    return RT_EOK;
}
#endif 

int list_slot()
{
   // slot_show_info();
    return RT_EOK;
}

int slot_mng_init()
{
    g_pstSlotInfo = (slot_mem_info*)malloc(MAX_SLOT_NUM * sizeof(struct slot_mem_info));
    if(!g_pstSlotInfo)
    {
        debug_to_rs485_1("g_pstSlotInfo malloc err!\n");
        return RT_ERROR;
    }
    
    memset(g_pstSlotInfo, 0, MAX_SLOT_NUM * sizeof(struct slot_mem_info));
    
    return slot_reg_dev(FIX_SLOT_NUM, USR_M100);
}


void protocol_extender_recv(uint8_t *buf, uint16_t len, uint8_t *recv, uint8_t flag)
{
    //slave_port_t port_t = NULL;
    slave_port_type *ptr_slave_port_recv=0;
    uint8_t idr = 0;
            




    if(len < PROTOCOL_ADD_LEN)
    {
        if(log_flag)
           {
            debug_to_rs485_1("extender recv len err!\n");
           }
        return;
    }


    for(int i = 0; i < len - PROTOCOL_ADD_LEN; i++)
    {

        idr = ~buf[i + 1];

        if(buf[i] == idr)
        {

            
            
            
            if(buf[i + 4] > len - PROTOCOL_ADD_LEN)
                continue;



            ptr_slave_port_recv = protocol_extender_analysis(&buf[i], buf[i + 4] + PROTOCOL_ADD_LEN);//有malloc，处理完需要释放
            if(ptr_slave_port_recv == RT_NULL)
            {
                continue;
            }
            if(ptr_slave_port_recv->len == 1)
            {
            
             
            //memset(extender_recvdebug_buff,0,100);
            


                g_pstSlotInfo[ptr_slave_port_recv->id].uiRunType = dev_get_type_by_extender(*(ptr_slave_port_recv->data));


            }
            else
            {
            
             
            //memset(extender_recvdebug_buff,0,100);
            



                modbus_unpack(ptr_slave_port_recv->data,ptr_slave_port_recv->len, recv, flag);
            }
            
            g_pstSlotInfo[ptr_slave_port_recv->id].ackTick = millis();
            
            i = i + buf[i + 4] + PROTOCOL_ADD_LEN - 1;  //直接跳到下一帧数据进行判断
            
            if(ptr_slave_port_recv->data)
                free(ptr_slave_port_recv->data);

            if(ptr_slave_port_recv)
            {
                free(ptr_slave_port_recv);
                ptr_slave_port_recv = NULL;
            }                
        }
    }//end for   

    return;
}



void poll_send_recv(uint8_t id, DEV_TYPE uiDevType, uint32_t timeout)
{
    uint8_t send_data[128] = {0};
    
   
    uint8_t register_len_di = 0;
    uint8_t register_len_ai = 0;
   
    uint8_t *send_buf;
   



    switch(uiDevType)
    {
        case USR_IO4040:
            register_len_di = 4;
          
            break;
        case USR_IO0440:
            register_len_ai = 8;
       
            break;
        case USR_IO0080:
            
            break;
        case USR_IO8000:
            register_len_di = 8;
            break;
        case USR_IO0800:
            register_len_ai = 16;
            break;
        case USR_IO0404:
            register_len_ai = 8;
            

            break;
        default:
            break;
    }

    send_buf = modbus_package(id, IO_READ_HOLDING_REGISTER, EXT_VER_REGISTER_ADDR, EXT_VER_REGISTER_LEN, NULL, send_data);      //查版本信息
    send_buf = modbus_package(id, IO_READ_INPUT, EXT_DI_SWITCH_START, register_len_di, NULL, send_buf);                         //查DI数据
    //send_buf = modbus_package(id, IO_READ_COIL, EXT_DO_SWITCH_START, register_len_do, NULL, send_buf);                          //查DO数据
    send_buf = modbus_package(id, IO_READ_INPUT_REGISTER, EXT_AI_SWITCH_START, register_len_ai, NULL, send_buf);                //查AI数据


    protocol_extender_send((send_buf - send_data),id ,PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, timeout);

    return;
}


//                         00(设备地址)   9600（波特率）  8（数据位）       1（停止位）       0（校验位）
void extender_addr_set(uint8_t id, uint32_t buad, uint8_t datasize, uint8_t stopbit, uint8_t parity)
{
    uint16_t  crc;
    uint8_t addr_set_buf[16] = {0};
   

    addr_set_buf[0] = ADDR_CODED_BUS_PROTOCOL_HEAD;
    addr_set_buf[1] = id - 1;
    addr_set_buf[2] = id;
    addr_set_buf[3] = (buad & 0xFF000000) >> 24;
    addr_set_buf[4] = (buad & 0x00FF0000) >> 16;
    addr_set_buf[5] = (buad & 0x0000FF00) >> 8;
    addr_set_buf[6] = (buad & 0x000000FF);
    addr_set_buf[7] = datasize;
    addr_set_buf[8] = stopbit;
    addr_set_buf[9] = parity;

    crc = crc16((char *)&addr_set_buf[0], 10); //这个函数有问题，后面更换

    addr_set_buf[11] = (crc & 0xFF);
    addr_set_buf[10] = crc >> 8;

    uart7_send((unsigned char *)addr_set_buf, ADDR_CODED_BUS_DATA_LEN);

    return;
}

void slot_mng_status_get(uint8_t slot_num)
{
    if(g_pstSlotInfo[slot_num].uiRunType)
    {
        g_pstSlotInfo[slot_num].uiStatus = SLOT_STATUS_OK;
    }
    else
        g_pstSlotInfo[slot_num].uiStatus = SLOT_STATUS_NULL;

    return;
}

void do_update_status()
{

}

void usr_io_task(void *param) 
{
  
    uint8_t count = 0;
    uint8_t slot_num = 0;
    uint8_t init_flag[MAX_SLOT_NUM] = {0};
    uint8_t data[1 + PROTOCOL_ADD_LEN + 1] = {0};

    while(!init_flag[0])
    {
        init_flag[0] = 1;
        extender_addr_set(EXT_ADDR, EXT_BUAD, EXT_DATASIZE, EXT_STOPBIT, EXT_PARITY);

        for(slot_num = MIN_SLOT_NUM; slot_num < MAX_SLOT_NUM; slot_num++)
        {
            slot_mng_status_get(slot_num);
            init_flag[slot_num]  = 1;

            init_flag[0] = init_flag[0] && init_flag[slot_num];
        }
        //reload_dog_in_usrlib();
        //vTaskDelay(1000 / portTICK_RATE_MS);  //挂起1s
        delay(2000);
       // delay(500);//从2秒改成500毫秒没有影响，那就维持2秒吧
    }

    while(1)
    {        

        if(count > 4)
        {
            extender_addr_set(EXT_ADDR, EXT_BUAD, EXT_DATASIZE, EXT_STOPBIT, EXT_PARITY);
            //此行屏蔽测试ao输出电压不对。
            for(slot_num = MIN_SLOT_NUM; slot_num < MAX_SLOT_NUM; slot_num++)
            {
                // rt_mutex_take(ext_data_get, RT_WAITING_FOREVER);
                memset(data, 0, PROTOCOL_ADD_LEN + 2);
                if(g_pstSlotInfo[1].uiStatus != SLOT_STATUS_OK)
                {
                    //这个地方更新查询扩展机状态会影响 g_pstSlotInfo[slot_num].uiStatus这个变量。

 
                    // reload_dog_in_usrlib();
  
                    protocol_extender_send(STATE_GET_DATA_LEN, slot_num, PRO_ORDER_STATE_GET, data, EXT_POLL_TIMEOUT);  //查询扩展机状态
                    gv_recv_flag = 0;
                }
                    
                
            }
            count = 0;
        }
        
        for(slot_num = MIN_SLOT_NUM; slot_num < MAX_SLOT_NUM; slot_num++)
        {

            slot_mng_status_get(slot_num);  //更新扩展机在线状态            

            if(g_pstSlotInfo[slot_num].uiStatus != SLOT_STATUS_OK)
                continue;

            if(led_sta_flag[slot_num] != LED_WORT_MODE)
            {
               // ext_status_led_set(slot_num, LED_OFF);  //扩展机
                led_sta_flag[slot_num] = LED_WORT_MODE;
            }
            poll_send_recv(slot_num, g_pstSlotInfo[slot_num].uiRunType, EXT_POLL_TIMEOUT);

             delay(100); // 如果300 毫秒开机要6个255后才有数据 改成100毫秒开机后4个255 有数据。
            
        }
        
        count ++;
        //vTaskDelay(100 / portTICK_RATE_MS);  //挂起1s
        delay(100);//这里改成100只要2个255了；
    }

}

#define BUFFER_LEN_MAX 256
uint8_t recv_flag = 0;
uint8_t ext_recv_data[BUFFER_LEN_MAX] = {0};

enum ext_data_get_flag
{
    EXT_DATA_GET_POLL = 0,     //轮询
    EXT_DATA_GET_ONLY = 1      //单条数据查询
};
int gvu16_receivelinebufindex=0;


void usr_recv_task(void *xTask2) 
{
unsigned char uart3_recvbuff[BUFFER_LEN_MAX ] = {0};
unsigned int recvbuffdata_length=0;
  while (1) {
    int availnum = 0;
    delay(100);  //固定周期性休息一下但时间不太长
    
    //RS485.receive();
    // delay(1000);
      pinMode(TX2DEPIN, OUTPUT);
	digitalWrite(TX2DEPIN, LOW);
    availnum = Serial2.available();
    if (availnum>0)
    {

      delay(200);
    }


    if (availnum) 
    {

      for (int i = 0; i < availnum; i++) 
      {
        if (gvu16_receivelinebufindex < BUFFER_LEN_MAX) {
          uart3_recvbuff[gvu16_receivelinebufindex] = Serial2.read();


          gvu16_receivelinebufindex++;
         // reload_dog_in_usrlib();
        }
        recvbuffdata_length=gvu16_receivelinebufindex+1;



      }  //for
    

     delay(200);
    if (recvbuffdata_length > PROTOCOL_ADD_LEN) 
        {
          protocol_extender_recv((uint8_t *)uart3_recvbuff, recvbuffdata_length, ext_recv_data, gv_recv_flag);
                gvu16_receivelinebufindex=0;
                memset(uart3_recvbuff,0,BUFFER_LEN_MAX);
                gv_recv_flag = 0;
        }
    }    //if(availnum)
    else 
    {
      ////485没有检测到数据接下来的时间大概率也是没有数据，这个读数据的task好好休息一会儿。
      delay(1000);
    }
  }  //while(1)
}




////////////////////////////////////////////



void USR_IO::init(){
    const size_t stack_size = 10*1024;
    
    uint8_t slot_num;
   
    
     logprint_serial_in_userd_Semaphore = xSemaphoreCreateBinary();




    port_mng_init();//端口初始化 这行代码，会开机死机，为什么

    slot_mng_init(); //槽位初始化
    //slot_reg_dev(FIX_SLOT_NUM, USR_M100);

    for (slot_num = MIN_SLOT_NUM; slot_num < MAX_SLOT_NUM; slot_num++)
    {
           // slot_reg_dev(slot_num, USR_IO0404);//regslt这个值我该怎么赋值？用DEV_TYPE这个枚举类型里面固定一个值吗？
          //  slot_reg_dev(slot_num, regslt);//regslt这个值我该怎么赋值？用DEV_TYPE这个枚举类型里面固定一个值吗？
    }
    //slot_reg_dev(0, USR_M100);

//如果demoai功能需要注册成USR_IO0404，那这样是不是根据不同的demo让代码走对应的case了
    // slot_reg_dev(1, USR_IO0404 );
    // slot_reg_dev(2, USR_IO0404);
    // slot_reg_dev(3, USR_IO0404);

     for(int i=0;i<MAX_SLOT_NUM;i++)
         for(int j=0;j<8;j++)
         {
            di_status[i][j]=-1;
         }

    slot_reg_dev(1, usrio_module_slot1_usrio_type);
    slot_reg_dev(2, usrio_module_slot2_usrio_type);
    slot_reg_dev(3, usrio_module_slot3_usrio_type);


    Serial.begin(EXT_CONFIG_BUAD);

     extender_addr_set(EXT_ADDR, EXT_BUAD, EXT_DATASIZE, EXT_STOPBIT, EXT_PARITY);
    Serial2.begin(EXT_BUAD, SERIAL_8N1, RX2PIN, TX2PIN);

    xTaskCreatePinnedToCore(&usr_io_task, "usr_io_task", stack_size, NULL, 5, NULL, 1);
    xTaskCreatePinnedToCore(&usr_recv_task, "usr_recv_task", stack_size, NULL, 5, NULL, 1);
}

void USR_IO::do_set(char slot_num, char io_num, char level)
{
    uint8_t *send_buf;

    uint8_t do_regisrg_len = 0;
    uint8_t send_data[16] = {0};
    uint16_t u16do_modbus_data[4] = {0};

    if(SLOT_STATUS_OK == g_pstSlotInfo[slot_num].uiStatus)
    {
        if(level == LED_ON)
            do_status_set[slot_num] |= (1 << (io_num - 1));
        else
            do_status_set[slot_num] &= ~(1 << (io_num - 1));
        
        if(g_pstSlotInfo[slot_num].uiRunType == USR_IO0080)
        {
            do_regisrg_len = 8;
        }
        else
        {
            do_regisrg_len = 4;
        }
        u16do_modbus_data[0]=do_status_set[slot_num];
        send_buf = modbus_package(slot_num, 0x0f, EXT_DO_SWITCH_START, do_regisrg_len, u16do_modbus_data, send_data);
        #if 0
         memset(do_debug_buf,0,100);
         snprintf(do_debug_buf,100,"modbus_package->\n");
         debug_to_rs485_1(do_debug_buf);
         for (int i=0;i<10;i++)
         {
                memset(do_debug_buf,0,100);
                //send_data[i]=i;
                snprintf(do_debug_buf,100,"0x%02x  ",send_data[i]);
                debug_to_rs485_1(do_debug_buf);
                if (i==9)
                {                 
                  debug_to_rs485_1("\n");  
                }

         }
        #endif
        protocol_extender_send((send_buf - send_data), slot_num ,PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, EXT_POLL_TIMEOUT);
        #if 0
         memset(do_debug_buf,0,100);
         snprintf(do_debug_buf,100,"protocol_extender_send->\n");
         debug_to_rs485_1(do_debug_buf);
         for (int i=0;i<7;i++)
         {
                memset(do_debug_buf,0,100);
                
                snprintf(do_debug_buf,100,"0x%02x ",send_data[i]);
                debug_to_rs485_1(do_debug_buf);
                if (i==6)
                {                 
                  debug_to_rs485_1("\n");  
                }

         }
        #endif
        //delay(500);
    }
}

#define VOLT_MODE       1
#define CURRENT_MODE    0
#define USER_IOAO_4ma 0x457a0000
#define USER_IOAO_20ma 0x469c4000
void USR_IO::ao_config(char slot_num, char io_num, char mode)
{
    uint8_t *send_buf;
    uint16_t regval=0;
    uint8_t ao_regisrg_len = 0;
    uint8_t send_data[30] = {0};
    uint16_t u16_aomode[2] = {0};
   

    g_pstSlotInfo[slot_num].uiStatus=SLOT_STATUS_OK;
    if(SLOT_STATUS_OK == g_pstSlotInfo[slot_num].uiStatus)
    {
        if(mode == VOLT_MODE)
            regval  = 0x0001;
        else
             regval  = 0x0004;
        
        ao_regisrg_len=0;
        
        u16_aomode[0]=regval;
        //IO_WRITE_REGISTER对应写单个寄存器，长度这个域用不到，随便写个值，组报文的时候或略的。
        
        //#define IO_WRITE_REGISTER               0x06        // 写单个寄存器（写单个寄存器）
       // extender_addr_set(EXT_ADDR, EXT_BUAD, EXT_DATASIZE, EXT_STOPBIT, EXT_PARITY);
       // delay(1000);
        //send_buf = modbus_package(slot_num, IO_WRITE_REGISTER, 0x0094, ao_regisrg_len, u16_aomode, send_data);
        send_buf = modbus_package(slot_num, IO_WRITE_REGISTER, EXT_AO_MODE, ao_regisrg_len, u16_aomode, send_data);
        #if 0
         memset(ao_debug_buf,0,100);
         for (int i=0;i<10;i++)
         {
            memset(ao_debug_buf,0,100);
            snprintf(ao_debug_buf,100,"0x%02x  ",send_data[i]);
            debug_to_rs485_1(ao_debug_buf);
            if (i==9)
            { 
                debug_to_rs485_1("\n");  
            }
         }
        #endif
        protocol_extender_send((send_buf - send_data), slot_num ,PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, EXT_POLL_TIMEOUT);
        delay(500);
    }
}//end ao_config

void USR_IO::ao_current_set(char slot_num, char io_num, char current_value)
{
    uint8_t *send_buf;
    uint8_t ao_regisrg_len = 0;
    uint8_t send_data[20] = {0};
    uint16_t ao_current_4ma[2]={0x457a,0x0};
    uint16_t ao_current_5ma[2]={0x459c,0x4000};
    uint16_t ao_current_10ma[2]={0x461c,0x4000};
    uint16_t ao_current_20ma[2]={0x469c,0x4000};
    
char ao_debug_buf[100];
    g_pstSlotInfo[slot_num].uiStatus=SLOT_STATUS_OK;
   // if(SLOT_STATUS_OK == g_pstSlotInfo[slot_num].uiStatus)
    {
        ao_regisrg_len=2;//规格书上看固定是2 不知道是否这样理解
        switch(current_value)
        {
            case 4:
                send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_current_4ma, send_data);
            break;
            
            case 5:
                send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_current_5ma, send_data);
            break;
            
            case 10:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_current_10ma, send_data);
            break;
            
            case 20:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_current_20ma, send_data);
            break;

        }


        
        #if 1
         memset(ao_debug_buf,0,100);
         for (int i=0;i<10;i++)
         {
                memset(ao_debug_buf,0,100);
                snprintf(ao_debug_buf,100,"0x%02x  ",send_data[i]);
                debug_to_rs485_1(ao_debug_buf);
                if (i==9)
                {
                  debug_to_rs485_1("\n");  
                }

         }
        
        #endif
        protocol_extender_send((send_buf - send_data), slot_num ,PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, EXT_POLL_TIMEOUT);
        delay(500);
    }
}// end ao_current_set
void USR_IO::ao_volt_set(char slot_num, char io_num, char volt_level)
{
    uint8_t *send_buf;
   // 
    uint8_t ao_regisrg_len = 0;
    uint8_t send_data[20] = {0};
    uint16_t ao_volt_0v[2]={0,0x0};
    uint16_t ao_volt_3v[2]={0x453b,0x8000};
    uint16_t ao_volt_6v[2]={0x45bb,0x8000};
    uint16_t ao_volt_10v[2]={0x461c,0x4000};

    //IO0440 4DO 4AI
    //IO0080 8DO
    //IO8000 8DI
   
    //if(SLOT_STATUS_OK == g_pstSlotInfo[slot_num].uiStatus)
    {
        ao_regisrg_len=2;//规格书上看固定是2 不知道是否这样理解
        //#define IO_WRITE_REG_ANY                0x10         // 写多个寄存器（写多个寄存器）
       
        switch(volt_level)
        {

      
        case 0:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_volt_0v, send_data);
            
        break;

        case 3:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_volt_3v, send_data);
            
        break;
        case 6:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_volt_6v, send_data);
            
        break;

        case 10:
            send_buf = modbus_package(slot_num, IO_WRITE_REG_ANY, EXT_AO_SWITCH_START, ao_regisrg_len, ao_volt_10v, send_data);
            
        break;
    }    
        #if 0
         memset(ao_debug_buf,0,100);
         for (int i=0;i<10;i++)
         {
                memset(ao_debug_buf,0,100);
                snprintf(ao_debug_buf,100,"0x%02x  ",send_data[i]);
                debug_to_rs485_1(ao_debug_buf);
                if (i==9)
                {
                  debug_to_rs485_1("\n");  
                }

         }
        
        #endif
        protocol_extender_send((send_buf - send_data), slot_num ,PRO_ORDER_RESOURCES_GET, (uint8_t *)send_data, EXT_POLL_TIMEOUT);
        delay(500);
    }
}// end ao_volt_set

// char USR_IO::do_get(char slot_num, char io_num)
// {

//     return 1;
// }

char USR_IO::di_get(char slot_num, char io_num)
{
    if(slot_num >= MAX_SLOT_NUM)
    {
        return -1;
    }

    return di_status[slot_num][io_num - 1];
}

float USR_IO::ai_get(char slot_num, char io_num)
{
    if(slot_num >= MAX_SLOT_NUM)
    {
        return -1;
    }

//return ai_val[io_num - 1];
    return ai_status[slot_num][io_num - 1];
}

void USR_IO::log_on(char log){
    log_flag = log;
}

void USR_IO::print()
{
    int i;
	char aRunName[MAX_DEV_NAME_LEN] = {0};
    char buf[128] = {0};

    if(!log_flag)
    {
        return;
    }

	sprintf(buf, "%10s %10s %10s %13s\r\n", "slotid", "runtype", "status", "ver");
    debug_to_rs485_1(buf);
	sprintf(buf, "%s", " ---------  ---------   --------   ------------------\r\n");
    debug_to_rs485_1(buf);

	for(i = FIX_SLOT_NUM; i < MAX_SLOT_NUM; i++)
	{
		dev_get_name_by_type(g_pstSlotInfo[i].uiRunType, aRunName);			
		sprintf(buf, "%10d %10s %10d %21s\r\n", i, aRunName, g_pstSlotInfo[i].uiStatus, g_pstSlotInfo[i].ext_ver);
        debug_to_rs485_1(buf);
    }
}

void USR_IO::debug_list_slot(void)
{
    list_slot();
}



void USR_IO:: slot_reg_device_id(unsigned short slot1device,unsigned short slot2device,unsigned short slot3device)
{
    usrio_module_slot1_usrio_type=slot1device;
    usrio_module_slot2_usrio_type=slot2device;
    usrio_module_slot3_usrio_type=slot3device;
}