/***************************************************************************************************
                                    ExploreEmbedded Copyright Notice    
****************************************************************************************************
 * File:   AWS_IOT.cpp
 * Version: 1.0
 * Author: ExploreEmbedded
 * Website: http://www.exploreembedded.com/wiki
 * Description: ESP32  Arduino library for AWS IOT.
 
This code has been developed and tested on ExploreEmbedded boards.  
We strongly believe that the library works on any of development boards for respective controllers. 
Check this link http://www.exploreembedded.com/wiki for awesome tutorials on 8051,PIC,AVR,ARM,Robotics,RTOS,IOT.
ExploreEmbedded invests substantial time and effort developing open source HW and SW tools, to support consider buying the ExploreEmbedded boards.
 
The ExploreEmbedded libraries and examples are licensed under the terms of the new-bsd license(two-clause bsd license).
See also: http://www.opensource.org/licenses/bsd-license.php

EXPLOREEMBEDDED DISCLAIMS ANY KIND OF HARDWARE FAILURE RESULTING OUT OF USAGE OF LIBRARIES, DIRECTLY OR
INDIRECTLY. FILES MAY BE SUBJECT TO CHANGE WITHOUT PRIOR NOTICE. THE REVISION HISTORY CONTAINS THE INFORMATION 
RELATED TO UPDATES.
 

Permission to use, copy, modify, and distribute this software and its documentation for any purpose
and without fee is hereby granted, provided that this copyright notices appear in all copies 
and that both those copyright notices and this permission notice appear in supporting documentation.
**************************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include "system_parameter.h"
#include <parameter_config.h>
 //#include "parameter_eeprom_function.h"

#ifdef CONFIG_MACOS_ENABLE
#include <sys/select.h>
#endif
#include <EEPROM.h>








static const char *TAG = "system_parameter";
extern void EEPROM_set_M100SN_length(unsigned char length) ;
#define M100_EEPROM_SIZE 300
unsigned char EEPROMRAMBUF[M100_EEPROM_SIZE];
void refreshEEPROMRAM(void) 
{
  EEPROM.commit();
  for (int i = 0; i < M100_EEPROM_SIZE; i++) {
    EEPROMRAMBUF[i] = EEPROM.read(i);
  }
  EEPROM.commit();
}

void update_RAM_toEEPROM(void) 
{
    for (int i = 0; i < M100_EEPROM_SIZE; i++) 
    {
        EEPROM.write(i, EEPROMRAMBUF[i]);
    }
    EEPROM.commit();
}
void writetoEEPROMRAM(unsigned char index, unsigned char val) 
{
  if (index < M100_EEPROM_SIZE) {
    EEPROMRAMBUF[index] = val;
  }
}
unsigned char readfromEEPROM_cache(unsigned char index) 
{
  if (index < M100_EEPROM_SIZE) {
    return EEPROMRAMBUF[index];
  } else {
    return 0;
  }
}
static unsigned char EEPROM_getTOP_ADDR_VALUE(void) 
{
  return readfromEEPROM_cache(TOP_ADDR);
}
 void EEPROM_set_TOP_ADDR_VALUE(unsigned char value) 
{
  refreshEEPROMRAM();
  writetoEEPROMRAM(TOP_ADDR, value);
  update_RAM_toEEPROM();
}

 void EEPROM_set_M100SN_length(unsigned char length) 
{
  refreshEEPROMRAM();
  writetoEEPROMRAM(M100_SN_LENGTH_ADR, length);
  update_RAM_toEEPROM();
}


static unsigned char EEPROM_getM100SN_length(void) 
{
  return readfromEEPROM_cache(M100_SN_LENGTH_ADR);
}


static void EEPROM_set_M100SNr_string(char arr[], int length) 
{
  int i, intaddr = 0;
  if (length > M100_SN_LENGTH_MAX) {
    length = M100_SN_LENGTH_MAX;
  }
  refreshEEPROMRAM();

  for (i = 0; i < length; i++) 
  {
    intaddr = M100SN_ADR;
    writetoEEPROMRAM((intaddr + i), 0);
  }
  for (i = 0; i < length; i++) {
    intaddr = M100SN_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }

  update_RAM_toEEPROM();
}

static void EEPROM_set_set_base_mac_length(unsigned char length) 
{
  refreshEEPROMRAM();
  writetoEEPROMRAM(M100_BASEMAC_LENGTH_ADR, length);
  update_RAM_toEEPROM();
}


static void EEPROM_set_base_mac_string(char arr[], unsigned char length) 
{
  int i, intaddr = 0;
  refreshEEPROMRAM();
  //先把保存mac这段长度清0避免乱码
  for (i = 0; i < BASE_MAC_LENGTH; i++) {
    intaddr = CUSTOM_BASE_MAC;
    writetoEEPROMRAM((intaddr + i), 0);
  }
  if (length > BASE_MAC_LENGTH) {
    length = BASE_MAC_LENGTH;
  }
  for (i = 0; i < length; i++) {
    intaddr = CUSTOM_BASE_MAC;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}
static void EEPROM_setpoweronwifistatus(unsigned char flag) 
{
  //refreshEEPROMRAM();
  writetoEEPROMRAM(WIFI_mode_ADDR, flag);
  update_RAM_toEEPROM();
}




static void EEPROM_set_SMALLCURRENT_ADC(int small_adc_value) 
{
  refreshEEPROMRAM();
  Serial.printf("line 450 <EEPROM_set_SMALLCURRENT_ADC> small_adc_value=%d adc_value=%x\n", small_adc_value, small_adc_value);
  unsigned char LOW8_bit = small_adc_value & 0xff;
  unsigned char HIGH8_bit = (small_adc_value >> 8) & 0xff;
  writetoEEPROMRAM(SMALLCURRENT_ADC_LOW8BIT_ADR, LOW8_bit);
  writetoEEPROMRAM(SMALLCURRENT_ADC_HIGH8BIT_ADR, HIGH8_bit);
  Serial.printf("\nline 455 <EEPROM_set_SMALLCURRENT_ADC> LOW8_bit=%d LOW8_bit=%x\n", LOW8_bit, LOW8_bit);
  Serial.printf("\nline 456 <EEPROM_set_SMALLCURRENT_ADC> HIGH8_bit=%d HIGH8_bit=%x\n", HIGH8_bit, HIGH8_bit);
  update_RAM_toEEPROM();
}

static void EEPROM_set_BIGCURRENT_ADC(int big_adc_value) 
{
  refreshEEPROMRAM();
  Serial.printf("line 457 <EEPROM_set_BIGCURRENT_ADC>big_adc_value=%d big_adc_value=0X%X\n", big_adc_value, big_adc_value);
  unsigned char LOW8_bit = big_adc_value & 0xff;
  unsigned char HIGH8_bit = (big_adc_value >> 8) & 0xff;
  writetoEEPROMRAM(BIGCURRENT_ADC_LOW8BIT_ADR, LOW8_bit);
  writetoEEPROMRAM(BIGCURRENT_ADC_HIGH8BIT_ADR, HIGH8_bit);
  update_RAM_toEEPROM();
}

static int EEPROM__getBIGCURRENT_ADC(void) 
{
  int retvalue = 0;
  unsigned char LOW8_bit;
  unsigned char HIGH8_bit;
  refreshEEPROMRAM();
  HIGH8_bit = readfromEEPROM_cache(BIGCURRENT_ADC_HIGH8BIT_ADR);
  LOW8_bit = readfromEEPROM_cache(BIGCURRENT_ADC_LOW8BIT_ADR);
  retvalue = (HIGH8_bit << 8) | LOW8_bit;
  //Serial.printf("line 491 <EEPROM__getBIGCURRENT_ADC>retvalue=%d big_adc_value=0X%X\n", retvalue, retvalue);
  return retvalue;
}


static int EEPROM__getSMALLCURRENT_ADC(void) 
{
  int retvalue = 0;
  refreshEEPROMRAM();
  retvalue = (readfromEEPROM_cache(SMALLCURRENT_ADC_HIGH8BIT_ADR)) << 8 | (readfromEEPROM_cache(SMALLCURRENT_ADC_LOW8BIT_ADR));
  return retvalue;
}

static void EEPROM_set_CURRENT_ADCINITIAL(void) 
{
  int big_adc_value;
  int small_adc_value;
  big_adc_value = 1121;
  unsigned char LOW8_bit;
  unsigned char HIGH8_bit;
  LOW8_bit = big_adc_value & 0xff;
  HIGH8_bit = (big_adc_value >> 8) & 0xff;
  refreshEEPROMRAM();
  writetoEEPROMRAM(BIGCURRENT_ADC_LOW8BIT_ADR, LOW8_bit);
  writetoEEPROMRAM(BIGCURRENT_ADC_HIGH8BIT_ADR, HIGH8_bit);

  small_adc_value = 284;

  LOW8_bit = small_adc_value & 0xff;
  HIGH8_bit = (small_adc_value >> 8) & 0xff;
  writetoEEPROMRAM(SMALLCURRENT_ADC_LOW8BIT_ADR, LOW8_bit);
  writetoEEPROMRAM(SMALLCURRENT_ADC_HIGH8BIT_ADR, HIGH8_bit);
  update_RAM_toEEPROM();
}


static void EEPROM_set_base_mac_length(unsigned char length) 
{
  refreshEEPROMRAM();
  writetoEEPROMRAM(M100_BASEMAC_LENGTH_ADR, length);
  update_RAM_toEEPROM();
}

static unsigned char EEPROM_getbase_mac_length(void) 
{
  return readfromEEPROM_cache(M100_BASEMAC_LENGTH_ADR);
}

static void EEPROM_set_modbustcpserver_ip_length(unsigned char length) 
{
  writetoEEPROMRAM(MODBUSTCPSERVER_IP_length_ADDR, length);
  update_RAM_toEEPROM();
}

static void EEPROM_set_tcpserver_ip_length(unsigned char length) 
{
  writetoEEPROMRAM(TCPSERVER_IP_length_ADDR, length);
  update_RAM_toEEPROM();
}


static void EEPROM_set_mqtt_server_ip_length(unsigned char length) 
{
  writetoEEPROMRAM(MQTTSERVER_IP_length_ADDR, length);
  update_RAM_toEEPROM();
}
static void EEPROM_set_mqtt_server_user_length(unsigned char length) 
{
  //refreshEEPROMRAM();
  writetoEEPROMRAM(MQTTSERVER_USER_length_ADDR, length);
  update_RAM_toEEPROM();
}

static void EEPROM_set_mqtt_server_pass_length(unsigned char length) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(MQTTSERVER_PASS_length_ADDR, length);
  update_RAM_toEEPROM();
}

static void EEPROM_set_tcpserver_port_length(unsigned char length) 
{
  //refreshEEPROMRAM();
  writetoEEPROMRAM(TCPSERVER_PORT_length_ADDR, length);
  update_RAM_toEEPROM();
}
static void EEPROM_setpoweronwifiSSIDlength(unsigned char length) 
{
 // refreshEEPROMRAM();
  if (length > WIFI_SSIDstring_length_MAX) 
  {
    length = WIFI_SSIDstring_length_MAX;
  }
  writetoEEPROMRAM(WIFI_SSIDlength_ADR, length);
  update_RAM_toEEPROM();
}

static void EEPROM_slot1_setusrio_type(unsigned short type) 
{
 // refreshEEPROMRAM();
  unsigned char LOW8_bit;
 unsigned char HIGH8_bit;
      LOW8_bit = type & 0xff;
  HIGH8_bit = (type >> 8) & 0xff;
  writetoEEPROMRAM(SLOT1_USRIO_TYPE_ADDR_LOW8BIT, LOW8_bit);
  writetoEEPROMRAM(SLOT1_USRIO_TYPE_ADDR_HIGH8BIT, HIGH8_bit);

 
  update_RAM_toEEPROM();
}


static unsigned short EEPROM_slot1_Getusrio_type(void) 
{
  //refreshEEPROMRAM();

  unsigned short retvalue=0;

  refreshEEPROMRAM();
  retvalue = (readfromEEPROM_cache(SLOT1_USRIO_TYPE_ADDR_HIGH8BIT)) << 8 | (readfromEEPROM_cache(SLOT1_USRIO_TYPE_ADDR_LOW8BIT));
  return retvalue;
}
static void EEPROM_slot2_setusrio_type(unsigned short type) 
{
 // refreshEEPROMRAM();
 unsigned char LOW8_bit;
 unsigned char HIGH8_bit;
  LOW8_bit = type & 0xff;
  HIGH8_bit = (type >> 8) & 0xff;
  writetoEEPROMRAM(SLOT2_USRIO_TYPE_ADDR_LOW8BIT, LOW8_bit);
  writetoEEPROMRAM(SLOT2_USRIO_TYPE_ADDR_HIGH8BIT, HIGH8_bit);

  update_RAM_toEEPROM();
}


static unsigned short EEPROM_slot2_Getusrio_type(void) 
{
  //refreshEEPROMRAM();
  unsigned short retvalue=0;

  refreshEEPROMRAM();
  retvalue = (readfromEEPROM_cache(SLOT2_USRIO_TYPE_ADDR_HIGH8BIT)) << 8 | (readfromEEPROM_cache(SLOT2_USRIO_TYPE_ADDR_LOW8BIT));
  return retvalue;
}
static void EEPROM_slot3_setusrio_type(unsigned short type) 
{
 // refreshEEPROMRAM();
  unsigned char LOW8_bit;
 unsigned char HIGH8_bit;
      LOW8_bit = type & 0xff;
  HIGH8_bit = (type >> 8) & 0xff;
  writetoEEPROMRAM(SLOT3_USRIO_TYPE_ADDR_LOW8BIT, LOW8_bit);
  writetoEEPROMRAM(SLOT3_USRIO_TYPE_ADDR_HIGH8BIT, HIGH8_bit);

  
  update_RAM_toEEPROM();
}


static unsigned short EEPROM_slot3_Getusrio_type(void) 
{
  //refreshEEPROMRAM();
  unsigned short retvalue=0;

  refreshEEPROMRAM();
  retvalue = (readfromEEPROM_cache(SLOT3_USRIO_TYPE_ADDR_HIGH8BIT)) << 8 | (readfromEEPROM_cache(SLOT3_USRIO_TYPE_ADDR_LOW8BIT));
  return retvalue;
}

static void EEPROM_setpoweronwifiSSIDstr(char arr[], int length) 
{
  
 // refreshEEPROMRAM();
  if (length > WIFI_SSIDstring_LENGTH_MAX) 
  {
    length = WIFI_SSIDstring_LENGTH_MAX;
  }

  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = WIFI_SSIDstringADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
   update_RAM_toEEPROM();
}
static void EEPROM_setwifi_passward_str(char arr[], int length) 
{
  
 // refreshEEPROMRAM();
  if (length > WIFI_PASSWARDstring_LENGTH_MAX) 
  {
    length = WIFI_PASSWARDstring_LENGTH_MAX;
  }


  for (int i = 0; i < length; i++) 
  {
    int intaddr = WIFI_PASSWARDstring_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
   update_RAM_toEEPROM();
}


static void EEPROM_setwifi_passward_length(unsigned char length) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(WIFI_PASSWARD_length_ADDR, length);
  update_RAM_toEEPROM();
}

static void EEPROM_set_modbustcpserver_ip_string(char arr[], int length) 
{
  
 // refreshEEPROMRAM();
  if (length > MODBUSTCPSERVERstring_length_MAX) 
  {
    length = MODBUSTCPSERVERstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = MODBUS_TCPSERVER_IPstring_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_tcpserver_ip_string(char arr[], int length) 
{
  
 // refreshEEPROMRAM();
  if (length > TCPSERVER_IPstring_length_MAX) 
  {
    length = TCPSERVER_IPstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = TCPSERVER_IPstring_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_tcpserver_port__string(char arr[], int length) 
{
  

  if (length > TCPSERVER_PORTstring_length_MAX) 
  {
    length = TCPSERVER_PORTstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = TCPSERVER_PORTstring_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_mqttserver_ip_string(char arr[], int length) 
{
  if (length > MQTTSERVER_IP_string_length_MAX) 
  {
    length = MQTTSERVER_IP_string_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = MQTTSERVER_IP_STRING_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_mqttserver_user_string(char arr[], int length) 
{
  if (length > MQTTSERVER_USER_length_MAX) 
  {
    length = MQTTSERVER_USER_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = MQTTSERVER_USER_STRING_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_mqtt_pass_string(char arr[], int length) 
{
  if (length > MQTTSERVER_PASS_length_MAX) 
  {
    length = MQTTSERVER_PASS_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = MQTTSERVER_PASS_STRING_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}



 static unsigned char EEPROM_GET_74HC595CHIP_VALUE(void) 
 {
   refreshEEPROMRAM();
   return readfromEEPROM_cache(CHIP_74HC595_VALUE_ADR);
 }

static unsigned char EEPROM_Getpoweron_wifi_status(void) 
{
  //refreshEEPROMRAM();
  return readfromEEPROM_cache(WIFI_mode_ADDR);
}
static unsigned char EEPROM_GetdeviceMode(void) 
{
  
  return readfromEEPROM_cache(deviceMode_ADDR);
}


static void EEPROM_getm100sn_string(char arr[], unsigned char length) 
{
  
  
  if (length > M100_SN_LENGTH_MAX) {
    length = M100_SN_LENGTH_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = M100SN_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}

static void EEPROM_getwifi_PASSWARD__string(char arr[], unsigned char length) 
{
  if (length > WIFI_PASSWARDstring_LENGTH_MAX) 
  {
    length = WIFI_PASSWARDstring_LENGTH_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
     int  intaddr = 0;
    intaddr = WIFI_PASSWARDstring_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}
static void EEPROM_get_tcpserver_port__string(char arr[], unsigned char length) 
{
  if (length > TCPSERVER_PORTstring_length_MAX) 
  {
    length = TCPSERVER_PORTstring_length_MAX;
  }
 
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = TCPSERVER_PORTstring_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}



static void EEPROM_getmqtt_SERVER_string(char arr[], unsigned char length) 
{
  if (length > MQTT_SERVER_IPstring_length_MAX) 
  {
    length = MQTT_SERVER_IPstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = MQTTSERVER_IP_STRING_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}

static void EEPROM_getmqtt_user_string(char arr[], unsigned char length) 
{
  if (length > MQTTSERVER_USER_length_MAX) 
  {
    length = MQTTSERVER_USER_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = MQTTSERVER_USER_STRING_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}

static void EEPROM_getmqtt_pass_string(char arr[], unsigned char length) 
{
  if (length > MQTTSERVER_PASS_length_MAX) 
  {
    length = MQTTSERVER_PASS_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
     int  intaddr = 0;
    intaddr = MQTTSERVER_PASS_STRING_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}


static void EEPROM_getpoweronwifiSSID_string(char arr[], unsigned char length) 
{
  
  //refreshEEPROMRAM();
  if (length > WIFI_SSIDstring_LENGTH_MAX) {
    length = WIFI_SSIDstring_LENGTH_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = WIFI_SSIDstringADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}

static void EEPROM_get_basemac_string(char arr[], unsigned char length) 
{
  
 // refreshEEPROMRAM();
  if (length > BASE_MAC_LENGTH) {
    length = BASE_MAC_LENGTH;
  }
  for (int i = 0; i < length; i++) 
  {
    int intaddr = 0;
    intaddr = CUSTOM_BASE_MAC;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}
static unsigned char EEPROM_get_modbusserver_ip_length(void) 
{
  return readfromEEPROM_cache(MODBUSTCPSERVER_IP_length_ADDR);
}

static unsigned char EEPROM_get_tcpserver_ip_length(void) 
{
  return readfromEEPROM_cache(TCPSERVER_IP_length_ADDR);
}
static void EEPROM_get_modbusserver_ip_string(char arr[], unsigned char length) 
{
  
 // refreshEEPROMRAM();
  if (length > MODBUSTCPSERVERstring_length_MAX) {
    length = MODBUSTCPSERVERstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = MODBUS_TCPSERVER_IPstring_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}


static void EEPROM_get_tcpserver_ip_string(char arr[], unsigned char length) 
{
  
 // refreshEEPROMRAM();
  if (length > TCPSERVER_IPstring_length_MAX) {
    length = TCPSERVER_IPstring_length_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = TCPSERVER_IPstring_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}

static unsigned char EEPROM_get_tcpserver_port_length(void) 
{
  return readfromEEPROM_cache(TCPSERVER_PORT_length_ADDR);
}

static unsigned char EEPROM_get_mqtt_server_ip_length(void) 
{
  return readfromEEPROM_cache(MQTTSERVER_IP_length_ADDR);
}
static unsigned char EEPROM_get_mqtt_server_user_length(void)
{
  return readfromEEPROM_cache(MQTTSERVER_USER_length_ADDR);
}
static unsigned char EEPROM_get_mqtt_server_pass_length(void) 
{
  return readfromEEPROM_cache(MQTTSERVER_PASS_length_ADDR);
}
static unsigned char EEPROM_getwifiSSIDlength(void) 
{
  return readfromEEPROM_cache(WIFI_SSIDlength_ADR);
}
static unsigned char EEPROM_getwifiPASSWARD_length(void) 
{
  return readfromEEPROM_cache(WIFI_PASSWARD_length_ADDR);
}

static void EEPROM__getM100_MUVER_string(char arr[], int length) 
{
  
 // refreshEEPROMRAM();
  if (length > M100_MUVER_LENGTH_MAX) 
  {
    length = M100_MUVER_LENGTH_MAX;
  }
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = M100_MUVER_ADR;
    arr[i] = readfromEEPROM_cache(intaddr + i);
  }
}
static unsigned char EEPROM_getM100_MUVER_length(void) 
{
 // refreshEEPROMRAM();
  return readfromEEPROM_cache(M100_MUVER_LENGTH_ADR);
}
//////////////////////////////////////////////////////////////////////////////////////////

int system_parameter::getM100_MUVER_string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM__getM100_MUVER_string(arr,length);
    return rc;
}


unsigned char system_parameter::getM100_MUVER_length(void)
{
   
    unsigned char M100_MUVER_length=0;
    M100_MUVER_length=EEPROM_getM100_MUVER_length();
return M100_MUVER_length;
}



int system_parameter::getwifi_password_length(void)
{
    
    unsigned char wifiPASSWARD_length=0;
    wifiPASSWARD_length=EEPROM_getwifiPASSWARD_length();
return wifiPASSWARD_length;
}
int system_parameter::getwifi_password__string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM_getwifi_PASSWARD__string(arr,length);
    return rc;
}

int system_parameter::getwifiSSIDlength(void)
{
    
    unsigned char wifiSSIDlength=0;
    wifiSSIDlength=EEPROM_getwifiSSIDlength();
return wifiSSIDlength;
}
int system_parameter::getpoweronwifiSSID_string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM_getpoweronwifiSSID_string(arr,length);
    return rc;
}

int system_parameter::get_mqtt_server_pass_length(void)
{
    
    unsigned char mqtt_server_pass_length=0;
    mqtt_server_pass_length=EEPROM_get_mqtt_server_pass_length();
return mqtt_server_pass_length;
}


int system_parameter::get_mqtt_server_user_length(void)
{
    
    unsigned char mqtt_server_user_length=0;
    mqtt_server_user_length=EEPROM_get_mqtt_server_user_length();
return mqtt_server_user_length;
}


int system_parameter::get_mqtt_server_ip_length(void)
{
    
    unsigned char mqtt_server_ip_length=0;
    mqtt_server_ip_length=EEPROM_get_mqtt_server_ip_length();
return mqtt_server_ip_length;
}


int system_parameter::get_tcpserver_port_length(void)
{
    
    unsigned char tcpserver_port_length=0;
    tcpserver_port_length=EEPROM_get_tcpserver_port_length();
return tcpserver_port_length;
}

int system_parameter::get_modbusserver_ip_length(void)
{
    //int rc=0;
    unsigned char modbusserver_ip_length=0;
    modbusserver_ip_length=EEPROM_get_modbusserver_ip_length();
return modbusserver_ip_length;
}


int system_parameter::get_tcpserver_ip_length(void)
{
    //int rc=0;
    unsigned char tcpserver_ip_length=0;
    tcpserver_ip_length=EEPROM_get_tcpserver_ip_length();
return tcpserver_ip_length;
}

int system_parameter::get_modbusserver_ip_string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM_get_modbusserver_ip_string(arr,length);
    return rc;
}
int system_parameter::get_tcpserver_ip_string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM_get_tcpserver_ip_string(arr,length);
    return rc;
}



int system_parameter::get_basemac_string(char arr[], unsigned char length)
{
    int rc=0;
    EEPROM_get_basemac_string(arr,length);
    return rc;
}





int system_parameter::get_mqtt_user_string(char arr[], unsigned char length)
{
    int rc=0;
    
    EEPROM_getmqtt_user_string(arr,length);
    return rc;
}

int system_parameter::get_mqtt_pass_string(char arr[], unsigned char length)
{
    int rc=0;
    
    EEPROM_getmqtt_pass_string(arr,length);
    return rc;
}

int system_parameter::get_mqtt_SERVER_string(char arr[], unsigned char length)
{
    int rc=0;
    
    EEPROM_getmqtt_SERVER_string(arr,length);
    return rc;
}

int system_parameter::get_tcpserver_port__string(char arr[], unsigned char length)
{
    int rc=0;
    
    EEPROM_get_tcpserver_port__string(arr,length);
    return rc;
}




int system_parameter::getm100sn_string(char arr[], unsigned char length)
{
    int rc=0;
    
    EEPROM_getm100sn_string(arr,length);
    return rc;
}



 unsigned char system_parameter::get_74HC595CHIP_VALUE(void)
{

     unsigned char u8_chip_data=0;
     u8_chip_data=EEPROM_GET_74HC595CHIP_VALUE();
     return u8_chip_data;
 }


int system_parameter::Getpoweron_wifi_status(void)
{
    
    unsigned char wifi_status=0;
    wifi_status=EEPROM_Getpoweron_wifi_status();
    return wifi_status;
}

int system_parameter::GetdeviceMode(void)
{
   
    unsigned char deviceMode=0;
    deviceMode=EEPROM_GetdeviceMode();
return deviceMode;
}

int system_parameter::set_tcpserver_port__string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_tcpserver_port__string(arr,length);
return rc;
}

int system_parameter::set_modbustcpserver_ip_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_modbustcpserver_ip_string(arr,length);
return rc;
}

int system_parameter::set_tcpserver_ip_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_tcpserver_ip_string(arr,length);
return rc;
}
int system_parameter::set_mqttserver_ip_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_mqttserver_ip_string(arr,length);
return rc;
}
int system_parameter::set_mqttserver_user_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_mqttserver_user_string(arr,length);
return rc;
}
int system_parameter::set_mqtt_pass_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_mqtt_pass_string(arr,length);
return rc;
}


int system_parameter::setwifi_password_length(unsigned char length)
{
    int rc=0;
    EEPROM_setwifi_passward_length(length);
return rc;
}


int system_parameter::setpoweronwifiSSIDlength(unsigned char SSIDlength)
{
    int rc=0;
    EEPROM_setpoweronwifiSSIDlength(SSIDlength);
return rc;
}
int system_parameter::setpoweronwifiSSIDstr(char arr[], int length)
{
    int rc=0;
    EEPROM_setpoweronwifiSSIDstr(arr,length);
return rc;
}
int system_parameter::setwifi_password_str(char arr[], int length)
{
    int rc=0;
    EEPROM_setwifi_passward_str(arr,length);
return rc;
}

int system_parameter::set_mqtt_server_ip_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_mqtt_server_ip_length(length);
    return rc;
}

int system_parameter::set_mqtt_server_user_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_mqtt_server_user_length(length);
return rc;
}


int system_parameter::set_mqtt_server_pass_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_mqtt_server_pass_length(length);
return rc;
}
int system_parameter::set_tcpserver_port_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_tcpserver_port_length(length);
return rc;
}




int system_parameter::set_sn_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_M100SN_length(length);
return rc;
}

int system_parameter::set_modbustcpserver_ip_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_modbustcpserver_ip_length(length);
return rc;
}

int system_parameter::set_tcpserver_ip_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_tcpserver_ip_length(length);
return rc;
}

int system_parameter::set_M100SNr_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_M100SNr_string(arr,  length);
return rc;
}

static void EEPROM_set_set_M100_MUVER_length(unsigned char length) 
{
  refreshEEPROMRAM();
  writetoEEPROMRAM(M100_MUVER_LENGTH_ADR, length);
  update_RAM_toEEPROM();
}


static void EEPROM_set_M100_MUVER_string(char arr[], int length) 
{
  
  if (length > M100_MUVER_LENGTH_MAX) {
    length = M100_MUVER_LENGTH_MAX;
  }
  refreshEEPROMRAM();
  for (int i = 0; i < length; i++) 
  {
    int  intaddr = 0;
    intaddr = M100_MUVER_ADR;
    writetoEEPROMRAM((intaddr + i), arr[i]);
  }
  update_RAM_toEEPROM();
}

static void EEPROM_set_74HC595_chipvalue(unsigned char outstatus_data) 
{
  //refreshEEPROMRAM();
  writetoEEPROMRAM(CHIP_74HC595_VALUE_ADR, outstatus_data);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_AT_MODE(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_Mode_AT_COMMAND);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_TCP_DTU_ONLY(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_TCP_DTU_ONLY);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_MQTT_DTU_ONLY(void) 
{
  //refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_MQTT_DTU_ONLY);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_HUAWEI_IOT_NOSSL(void) 
{
  
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_HUAWEI_IOT_NO_SSL);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_AWS_IOT(void) 
{
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_AWS_ONLY);
  update_RAM_toEEPROM();
}
static void EEPROM_setdeviceMode_MODBUSCONTROLIO_ONLY(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_MODBUSCONTROLIO);
  update_RAM_toEEPROM();
}
static void EEPROM_set_deviceMode_TH485PE_DEMO(void) 
{
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_TH485PE_DEMO);
  update_RAM_toEEPROM();
}

// static void EEPROM_set_deviceMode_USERIO_AO_DEMO(void) 
// {
//   writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_USERIO_AO_DEMO);
//   update_RAM_toEEPROM();
// }

// static void EEPROM_set_deviceMode_USRIO_DEMO(void) 
// {
//   writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_USRIO_DEMO);
//   update_RAM_toEEPROM();
// }



static void EEPROM_setdeviceMode_MODBUS_RTU(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_MODBUS_RTU);
  update_RAM_toEEPROM();
  // refreshEEPROMRAM();
}

static void EEPROM_setdeviceMode_MODBUS_TCP(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceMode_ADDR, DEVICE_MODE_MODBUS_TCP);
  update_RAM_toEEPROM();
  // refreshEEPROMRAM();
}

static void EEPROM_setdCONNECT_TO_NET_WITH_WIFIMODE(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceconnectMode_ADDR,CONNECT_TO_NET_SELECT_WIFI);
  //writetoEEPROMRAM(9,0);
  update_RAM_toEEPROM();
}


static void EEPROM_setCONNECT_TO_NET_WITH_ETHERNETMODE(void) 
{
 // refreshEEPROMRAM();
  writetoEEPROMRAM(deviceconnectMode_ADDR, CONNECT_TO_NET_SELECT_ETHERNET);
  //writetoEEPROMRAM(deviceconnectMode_ADDR, 1);
  update_RAM_toEEPROM();
}
static unsigned char EEPROM_GetconnectMode(void) 
{
  return readfromEEPROM_cache(deviceconnectMode_ADDR);
}

static void EEPROM_config_start(void) 
{
  if (!EEPROM.begin(M100_EEPROM_SIZE)) 
  {
    Serial.println("failed to initialise EEPROM");
  }
  refreshEEPROMRAM();

}

////////////////////////////////////////////////////////////////////////////////////////////////////////
int system_parameter::parameter_config_start(void)
{
    int rc=0;
    EEPROM_config_start();
return rc;
}
int system_parameter::set_deviceMode_AT_MODE(void)
{
    int rc=0;
    EEPROM_setdeviceMode_AT_MODE();
return rc;
}
int system_parameter::set_deviceMode_TCP_DTU_ONLY(void)
{
    int rc=0;
    EEPROM_setdeviceMode_TCP_DTU_ONLY();
return rc;
}
int system_parameter::set_deviceMode_MQTT_DTU_ONLY(void)
{
    int rc=0;
    EEPROM_setdeviceMode_MQTT_DTU_ONLY();
return rc=0;
}
int system_parameter::set_deviceMode_HUAWEI_IOT_NOSSL(void)
{
    int rc=0;
    EEPROM_setdeviceMode_HUAWEI_IOT_NOSSL();
return rc;
}
int system_parameter::set_deviceMode_AWS_IOT(void)
{
    int rc=0;
    EEPROM_setdeviceMode_AWS_IOT();
return rc;
}
int system_parameter::set_deviceMode_MODBUSCONTROLIO_ONLY(void)
{
    int rc=0;
    EEPROM_setdeviceMode_MODBUSCONTROLIO_ONLY();
return rc;
}


// int system_parameter::set_deviceMode_USERIO_AO_DEMO(void)
// {
//     int rc=0;
//     EEPROM_set_deviceMode_USERIO_AO_DEMO();
    
// return rc;
// }

// int system_parameter::set_deviceMode_USRIO_DEMO(void)
// {
//     int rc=0;
//     EEPROM_set_deviceMode_USRIO_DEMO();
    
// return rc=0;
// }


int system_parameter::set_deviceMode_TH485PE_DEMO(void)
{
    int rc=0;
    EEPROM_set_deviceMode_TH485PE_DEMO();
return rc;
}
void system_parameter::set_deviceMode_MODBUS_RTU(void)
{
   // int rc=0;
    EEPROM_setdeviceMode_MODBUS_RTU();
//return rc;
}

void system_parameter::set_deviceMode_MODBUS_TCP(void)
{
    //int rc=0;
    EEPROM_setdeviceMode_MODBUS_TCP();
//return rc;
}

int system_parameter::set_CONNECT_TO_NET_WITH_WIFIMODE(void)
{
    int rc=0;
    EEPROM_setdCONNECT_TO_NET_WITH_WIFIMODE();
return rc;
}
int system_parameter::set_CONNECT_TO_NET_WITH_ETHERNETMODE(void)
{
    int rc=0;
    EEPROM_setCONNECT_TO_NET_WITH_ETHERNETMODE();
return rc;
}


int system_parameter::GetconnectMode(void)
{
   
    unsigned char connectMode;
    connectMode=EEPROM_GetconnectMode();
return connectMode;
}


int system_parameter::set_M100basemac_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_base_mac_string(arr,  length);
return rc;
}
int system_parameter::set_74HC595_chipvalue(unsigned char outstatus_data)
{
    int rc=0;
    EEPROM_set_74HC595_chipvalue(outstatus_data);
return rc;
}


int system_parameter::set_basemac_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_set_base_mac_length(length);
return rc;
}
unsigned char system_parameter::get_basemac_length(void)
{
    
    unsigned char basemac_length=0;
    basemac_length=EEPROM_getbase_mac_length();
return basemac_length;
}




int system_parameter::set_m100muver_length(unsigned char length)
{
    int rc=0;
    EEPROM_set_set_M100_MUVER_length(length);
return rc;
}
int system_parameter::set_m100muver_string(char arr[], int length)
{
    int rc=0;
    EEPROM_set_M100_MUVER_string(arr,  length);
return rc;
}
int system_parameter::set_wifi_status(unsigned char wifimode)
{
    int rc=0;
    EEPROM_setpoweronwifistatus(wifimode);
return rc;
}
int system_parameter::get_wifi_status(void)
{
    
    unsigned char wifi_status=0;
    wifi_status=EEPROM_Getpoweron_wifi_status();
return wifi_status;
}


int system_parameter::set_wifissid_length(unsigned char length)
{
    int rc=0;
    EEPROM_setpoweronwifiSSIDlength(length);
return rc;
}

int system_parameter::set_slot1_usrio_type(unsigned short type)
{
    int rc=0;
    EEPROM_slot1_setusrio_type(type);
return rc;
}

unsigned short system_parameter::get_slot1_usrio_type(void)
{
     unsigned short rc=0;
    rc=EEPROM_slot1_Getusrio_type();
return rc;
}


int system_parameter::set_slot2_usrio_type(unsigned short type)
{
    int rc=0;
    EEPROM_slot2_setusrio_type(type);
return rc;
}

unsigned short system_parameter::get_slot2_usrio_type(void)
{
     unsigned short rc=0;
    rc=EEPROM_slot2_Getusrio_type();
return rc;
}


int system_parameter::set_slot3_usrio_type(unsigned short type)
{
    int rc=0;
    EEPROM_slot3_setusrio_type(type);
return rc;
}

unsigned short system_parameter::get_slot3_usrio_type(void)
{
     unsigned short type=0;
    type=EEPROM_slot3_Getusrio_type();
return type;
}


int system_parameter::set_current_initialadc(void)
{
    int rc=0;
    EEPROM_set_CURRENT_ADCINITIAL();
return rc;
}
int system_parameter::set_smallcurrent_adc(int adc_value)
{
    int rc=0;
    EEPROM_set_SMALLCURRENT_ADC(adc_value);
return rc;
}

int system_parameter::set_bigcurrent_adc(int adc_value)
{
    int rc=0;
    EEPROM_set_BIGCURRENT_ADC(adc_value);
return rc;
}


int system_parameter::get_smallcurrent_adc(void)
{
   
    int smalladc=0;
    smalladc=EEPROM__getSMALLCURRENT_ADC();
return smalladc;
}
int system_parameter::get_bigcurrent_adc(void)
{
   
    int bigadc=0;
    bigadc=EEPROM__getBIGCURRENT_ADC();
    return bigadc;
}

int system_parameter::parameter_refreshEEPROMRAM(void)
{
    int rc=0;
    refreshEEPROMRAM();
    return rc;
}


int system_parameter::get_sn_length(void)
{
   
    unsigned char length;
    length=EEPROM_getM100SN_length();
    
    return length;
}


